from typing import Any
from typing_extensions import TypeAlias

# Enable when pyasn1 gets stubs:
# from pyasn1.codec.ber.encoder import AbstractItemEncoder
_AbstractItemEncoder: TypeAlias = Any

CLASSES: Any

class LDAPBooleanEncoder(_AbstractItemEncoder):
    supportIndefLenMode: bool
    # Requires pyasn1 > 0.3.7
    def encodeValue(self, value, asn1Spec, encodeFun, **options): ...

customTagMap: Any
customTypeMap: Any

def compute_ber_size(data): ...
def decode_message_fast(message): ...
def decode_sequence(message, start, stop, context_decoders: Any | None = ...): ...
def decode_integer(message, start, stop, context_decoders: Any | None = ...): ...
def decode_octet_string(message, start, stop, context_decoders: Any | None = ...): ...
def decode_boolean(message, start, stop, context_decoders: Any | None = ...): ...
def decode_bind_response(message, start, stop, context_decoders: Any | None = ...): ...
def decode_extended_response(message, start, stop, context_decoders: Any | None = ...): ...
def decode_intermediate_response(message, start, stop, context_decoders: Any | None = ...): ...
def decode_controls(message, start, stop, context_decoders: Any | None = ...): ...
def ldap_result_to_dict_fast(response): ...
def get_byte(x): ...
def get_bytes(x): ...

DECODERS: Any
BIND_RESPONSE_CONTEXT: Any
EXTENDED_RESPONSE_CONTEXT: Any
INTERMEDIATE_RESPONSE_CONTEXT: Any
LDAP_MESSAGE_CONTEXT: Any
CONTROLS_CONTEXT: Any



from typing import ClassVar

import passlib.utils.handlers as uh

__all__ = ["mysql323"]

class mysql323(uh.StaticHandler):
    name: ClassVar[str]
    checksum_size: ClassVar[int]
    checksum_chars: ClassVar[str]

class mysql41(uh.StaticHandler):
    name: ClassVar[str]
    checksum_chars: ClassVar[str]
    checksum_size: ClassVar[int]



from typing import Any, ClassVar
from typing_extensions import final

from tree_sitter import Language

@final
class Node:
    @property
    def start_byte(self) -> int: ...
    @property
    def start_point(self) -> tuple[int, int]: ...
    @property
    def end_byte(self) -> int: ...
    @property
    def end_point(self) -> tuple[int, int]: ...
    @property
    def has_changes(self) -> bool: ...
    @property
    def has_error(self) -> bool: ...
    @property
    def id(self) -> int: ...
    @property
    def is_missing(self) -> bool: ...
    @property
    def is_named(self) -> bool: ...
    @property
    def child_count(self) -> int: ...
    @property
    def named_child_count(self) -> bool: ...
    @property
    def children(self) -> list[Node]: ...
    @property
    def named_children(self) -> list[Node]: ...
    @property
    def next_named_sibling(self) -> Node | None: ...
    @property
    def next_sibling(self) -> Node | None: ...
    @property
    def parent(self) -> Node | None: ...
    @property
    def prev_named_sibling(self) -> Node | None: ...
    @property
    def prev_sibling(self) -> Node | None: ...
    @property
    def text(self) -> bytes | Any: ...  # can be None, but annoying to check
    @property
    def type(self) -> str: ...
    def children_by_field_name(self, name: str) -> list[Node]: ...
    def children_by_field_id(self, __id: int) -> list[Node]: ...
    def field_name_for_child(self, __child_index: int) -> str: ...
    def child_by_field_id(self, __id: int) -> Node | None: ...
    def child_by_field_name(self, __name: str) -> Node | None: ...
    __hash__: ClassVar[None]  # type: ignore[assignment]
    def sexp(self) -> str: ...
    def walk(self) -> TreeCursor: ...
    def __eq__(self, other: object) -> bool: ...
    def __ne__(self, other: object) -> bool: ...
    # There are __ge__, __gt__, __le__, __lt__ methods but they always return False
    #
    #    >>> n
    #    <Node kind=call, start_point=(0, 0), end_point=(0, 14)>
    #    >>> n >= "", n <= "", n >= 0, n <= 0, n >= (0,0), n <= (0,0)
    #    (False, False, False, False, False, False)

@final
class Parser:
    # At runtime, Parser(1, 2, 3) ignores the arguments, but that's most likely buggy code
    def __init__(self) -> None: ...
    def parse(self, source: bytes, old_tree: Tree | None = ..., keep_text: bool = ...) -> Tree: ...
    def set_language(self, __language: Language) -> None: ...

@final
class Query:
    # start_point and end_point arguments don't seem to do anything
    # TODO: sync with
    # https://github.com/tree-sitter/py-tree-sitter/blob/d3016edac2c33ce647653d896fbfb435ac2a6245/tree_sitter/binding.c#L1304
    def captures(self, node: Node) -> list[tuple[Node, str]]: ...

@final
class Range:
    @property
    def start_byte(self) -> int: ...
    @property
    def end_byte(self) -> int: ...
    @property
    def start_point(self) -> tuple[int, int]: ...
    @property
    def end_point(self) -> tuple[int, int]: ...

@final
class Tree:
    @property
    def root_node(self) -> Node: ...
    @property
    def text(self) -> bytes | Any: ...  # technically ReadableBuffer | Any
    def edit(
        self,
        start_byte: int,
        old_end_byte: int,
        new_end_byte: int,
        start_point: tuple[int, int],
        old_end_point: tuple[int, int],
        new_end_point: tuple[int, int],
    ) -> None: ...
    def get_changed_ranges(self, new_tree: Tree) -> list[Range]: ...
    def walk(self) -> TreeCursor: ...

@final
class TreeCursor:
    @property
    def node(self) -> Node: ...
    def copy(self) -> TreeCursor: ...
    def current_field_name(self) -> str | None: ...
    def goto_first_child(self) -> bool: ...
    def goto_next_sibling(self) -> bool: ...
    def goto_parent(self) -> bool: ...



from _typeshed import Incomplete, StrOrBytesPath, StrPath
from collections.abc import Iterator
from typing import Any
from typing_extensions import TypeAlias

from pygments.lexer import Lexer, LexerMeta

_OpenFile: TypeAlias = StrOrBytesPath | int  # copy/pasted from builtins.pyi

def get_all_lexers(plugins: bool = ...) -> Iterator[tuple[str, tuple[str, ...], tuple[str, ...], tuple[str, ...]]]: ...
def find_lexer_class(name: str) -> LexerMeta | None: ...
def find_lexer_class_by_name(_alias: str) -> LexerMeta: ...
def get_lexer_by_name(_alias: str, **options: Any) -> Lexer: ...
def load_lexer_from_file(filename: _OpenFile, lexername: str = ..., **options: Any) -> Lexer: ...
def find_lexer_class_for_filename(_fn: StrPath, code: str | bytes | None = ...) -> LexerMeta | None: ...
def get_lexer_for_filename(_fn: StrPath, code: str | bytes | None = ..., **options: Any) -> Lexer: ...
def get_lexer_for_mimetype(_mime: str, **options: Any) -> Lexer: ...
def guess_lexer_for_filename(_fn: StrPath, _text: str, **options: Any) -> LexerMeta: ...
def guess_lexer(_text: str | bytes, **options: Any) -> Lexer: ...

# Having every lexer class here doesn't seem to be worth it
def __getattr__(name: str) -> Incomplete: ...



from typing import Any

JSON_OID: int
JSONARRAY_OID: int
JSONB_OID: int
JSONBARRAY_OID: int

class Json:
    adapted: Any
    def __init__(self, adapted, dumps: Any | None = ...) -> None: ...
    def __conform__(self, proto): ...
    def dumps(self, obj): ...
    def prepare(self, conn) -> None: ...
    def getquoted(self): ...

def register_json(
    conn_or_curs: Any | None = ...,
    globally: bool = ...,
    loads: Any | None = ...,
    oid: Any | None = ...,
    array_oid: Any | None = ...,
    name: str = ...,
): ...
def register_default_json(conn_or_curs: Any | None = ..., globally: bool = ..., loads: Any | None = ...): ...
def register_default_jsonb(conn_or_curs: Any | None = ..., globally: bool = ..., loads: Any | None = ...): ...



from typing import IO, Any, AnyStr

__version__: str

def encode(
    obj: Any,
    ensure_ascii: bool = ...,
    double_precision: int = ...,
    encode_html_chars: bool = ...,
    escape_forward_slashes: bool = ...,
    sort_keys: bool = ...,
    indent: int = ...,
) -> str: ...
def dumps(
    obj: Any,
    ensure_ascii: bool = ...,
    double_precision: int = ...,
    encode_html_chars: bool = ...,
    escape_forward_slashes: bool = ...,
    sort_keys: bool = ...,
    indent: int = ...,
) -> str: ...
def dump(
    obj: Any,
    fp: IO[str],
    ensure_ascii: bool = ...,
    double_precision: int = ...,
    encode_html_chars: bool = ...,
    escape_forward_slashes: bool = ...,
    sort_keys: bool = ...,
    indent: int = ...,
) -> None: ...
def decode(s: AnyStr, precise_float: bool = ...) -> Any: ...
def loads(s: AnyStr, precise_float: bool = ...) -> Any: ...
def load(fp: IO[AnyStr], precise_float: bool = ...) -> Any: ...

class JSONDecodeError(ValueError): ...



import xml.dom.minidom
from _typeshed import Incomplete, Self
from abc import abstractmethod
from collections.abc import Callable, Generator, Iterable, Sequence
from typing import Any, ClassVar, Protocol, TypeVar, overload
from typing_extensions import Literal

from docutils.transforms import Transformer

_N = TypeVar("_N", bound=Node)

class _DomModule(Protocol):
    Document: type[xml.dom.minidom.Document]

class Node:
    # children is initialized by the subclasses
    children: Sequence[Node]
    parent: Node | None
    source: str | None
    line: int | None
    document: document | None
    def __bool__(self) -> Literal[True]: ...
    def asdom(self, dom: _DomModule | None = ...) -> xml.dom.minidom.Element: ...
    # While docutils documents the Node class to be abstract it does not
    # actually use the ABCMeta metaclass. We still set @abstractmethod here
    # (although it's not used in the docutils implementation) because it
    # makes Mypy reject Node() with "Cannot instantiate abstract class".
    @abstractmethod
    def copy(self: Self) -> Self: ...
    @abstractmethod
    def deepcopy(self: Self) -> Self: ...
    @abstractmethod
    def pformat(self, indent: str = ..., level: int = ...) -> str: ...
    @abstractmethod
    def astext(self) -> str: ...
    def setup_child(self, child: Node) -> None: ...
    def walk(self, visitor: NodeVisitor) -> bool: ...
    def walkabout(self, visitor: NodeVisitor) -> bool: ...
    @overload
    def findall(
        self, condition: type[_N], include_self: bool = ..., descend: bool = ..., siblings: bool = ..., ascend: bool = ...
    ) -> Generator[_N, None, None]: ...
    @overload
    def findall(
        self,
        condition: Callable[[Node], bool] | None = ...,
        include_self: bool = ...,
        descend: bool = ...,
        siblings: bool = ...,
        ascend: bool = ...,
    ) -> Generator[Node, None, None]: ...
    @overload
    def traverse(
        self, condition: type[_N], include_self: bool = ..., descend: bool = ..., siblings: bool = ..., ascend: bool = ...
    ) -> list[_N]: ...
    @overload
    def traverse(
        self,
        condition: Callable[[Node], bool] | None = ...,
        include_self: bool = ...,
        descend: bool = ...,
        siblings: bool = ...,
        ascend: bool = ...,
    ) -> list[Node]: ...
    @overload
    def next_node(
        self, condition: type[_N], include_self: bool = ..., descend: bool = ..., siblings: bool = ..., ascend: bool = ...
    ) -> _N: ...
    @overload
    def next_node(
        self,
        condition: Callable[[Node], bool] | None = ...,
        include_self: bool = ...,
        descend: bool = ...,
        siblings: bool = ...,
        ascend: bool = ...,
    ) -> Node: ...
    def previous_sibling(self) -> Node | None: ...

class Element(Node):
    children: list[Node]
    def __init__(self, rawsource: str = ..., *children: Node, **attributes): ...
    def __len__(self) -> int: ...
    def __contains__(self, key: str | Node) -> bool: ...
    @overload
    def __getitem__(self, key: str) -> Any: ...
    @overload
    def __getitem__(self, key: int) -> Node: ...
    @overload
    def __getitem__(self, key: slice) -> list[Node]: ...
    @overload
    def __setitem__(self, key: str, item: Any) -> None: ...
    @overload
    def __setitem__(self, key: int, item: Node) -> None: ...
    @overload
    def __setitem__(self, key: slice, item: Iterable[Node]) -> None: ...
    def __delitem__(self, key: str | int | slice) -> None: ...
    def __add__(self, other: list[Node]) -> list[Node]: ...
    def __radd__(self, other: list[Node]) -> list[Node]: ...
    def __iadd__(self: Self, other: Node | Iterable[Node]) -> Self: ...
    def copy(self: Self) -> Self: ...
    def deepcopy(self: Self) -> Self: ...
    def pformat(self, indent: str = ..., level: int = ...) -> str: ...
    def astext(self) -> str: ...
    def __getattr__(self, __name: str) -> Incomplete: ...

class Text(Node, str):
    tagname: ClassVar[str]
    children: tuple[()]

    # we omit the rawsource parameter because it has been deprecated and is ignored
    def __new__(cls: type[Self], data: str) -> Self: ...
    def __init__(self, data: str) -> None: ...
    def shortrepr(self, maxlen: int = ...) -> str: ...
    def copy(self: Self) -> Self: ...
    def deepcopy(self: Self) -> Self: ...
    def pformat(self, indent: str = ..., level: int = ...) -> str: ...
    def astext(self) -> str: ...
    def rstrip(self, chars: str | None = ...) -> str: ...
    def lstrip(self, chars: str | None = ...) -> str: ...

class Structural: ...
class Root: ...

class document(Root, Structural, Element):
    transformer: Transformer
    def copy(self: Self) -> Self: ...
    def deepcopy(self: Self) -> Self: ...
    def pformat(self, indent: str = ..., level: int = ...) -> str: ...
    def astext(self) -> str: ...
    def __getattr__(self, __name: str) -> Incomplete: ...

class NodeVisitor:
    def __init__(self, document: document): ...
    def __getattr__(self, __name: str) -> Incomplete: ...

def __getattr__(name: str) -> Incomplete: ...



from typing import Any

standard_formatter: Any

def find_attribute_helpers(attr_type, name, custom_formatter): ...
def format_attribute_values(schema, name, values, custom_formatter): ...
def find_attribute_validator(schema, name, custom_validator): ...



from _typeshed import Incomplete

from docutils.languages import _LanguageModule
from docutils.nodes import document
from docutils.parsers.rst import Directive
from docutils.utils import SystemMessage

def register_directive(name: str, directive: type[Directive]) -> None: ...
def directive(
    directive_name: str, language_module: _LanguageModule, document: document
) -> tuple[type[Directive] | None, list[SystemMessage]]: ...
def __getattr__(name: str) -> Incomplete: ...



from collections.abc import Generator
from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Properties(Serialisable):
    locked: Any
    defaultSize: Any
    disabled: Any
    uiObject: Any
    autoFill: Any
    autoLine: Any
    altText: Any
    textHAlign: Any
    textVAlign: Any
    lockText: Any
    justLastX: Any
    autoScale: Any
    rowHidden: Any
    colHidden: Any
    __elements__: Any
    anchor: Any
    def __init__(
        self,
        locked: Any | None = ...,
        defaultSize: Any | None = ...,
        _print: Any | None = ...,
        disabled: Any | None = ...,
        uiObject: Any | None = ...,
        autoFill: Any | None = ...,
        autoLine: Any | None = ...,
        altText: Any | None = ...,
        textHAlign: Any | None = ...,
        textVAlign: Any | None = ...,
        lockText: Any | None = ...,
        justLastX: Any | None = ...,
        autoScale: Any | None = ...,
        rowHidden: Any | None = ...,
        colHidden: Any | None = ...,
        anchor: Any | None = ...,
    ) -> None: ...

class CommentRecord(Serialisable):
    tagname: str
    ref: Any
    authorId: Any
    guid: Any
    shapeId: Any
    text: Any
    commentPr: Any
    author: Any
    __elements__: Any
    __attrs__: Any
    height: Any
    width: Any
    def __init__(
        self,
        ref: str = ...,
        authorId: int = ...,
        guid: Any | None = ...,
        shapeId: int = ...,
        text: Any | None = ...,
        commentPr: Any | None = ...,
        author: Any | None = ...,
        height: int = ...,
        width: int = ...,
    ) -> None: ...
    @classmethod
    def from_cell(cls, cell): ...
    @property
    def content(self): ...

class CommentSheet(Serialisable):
    tagname: str
    authors: Any
    commentList: Any
    extLst: Any
    mime_type: str
    __elements__: Any
    def __init__(self, authors: Any | None = ..., commentList: Any | None = ..., extLst: Any | None = ...) -> None: ...
    def to_tree(self): ...
    @property
    def comments(self) -> Generator[Any, None, None]: ...
    @classmethod
    def from_comments(cls, comments): ...
    def write_shapes(self, vml: Any | None = ...): ...
    @property
    def path(self): ...



import threading
from _typeshed import Incomplete

import isapi.simple
from isapi import ExtensionError as ExtensionError, isapicon as isapicon
from win32event import INFINITE as INFINITE

ISAPI_REQUEST: int
ISAPI_SHUTDOWN: int

class WorkerThread(threading.Thread):
    running: bool
    io_req_port: Incomplete
    extension: Incomplete
    def __init__(self, extension, io_req_port) -> None: ...
    def call_handler(self, cblock) -> None: ...

class ThreadPoolExtension(isapi.simple.SimpleExtension):
    max_workers: int
    worker_shutdown_wait: int
    workers: Incomplete
    dispatch_map: Incomplete
    io_req_port: Incomplete
    def GetExtensionVersion(self, vi) -> None: ...
    def HttpExtensionProc(self, control_block): ...
    def TerminateExtension(self, status) -> None: ...
    def DispatchConnection(self, errCode, bytes, key, overlapped) -> None: ...
    def Dispatch(self, ecb) -> None: ...
    def HandleDispatchError(self, ecb) -> None: ...



from typing import Any

from ...types import Numeric
from .base import MSDialect, MSIdentifierPreparer

class _MSNumeric_pymssql(Numeric):
    def result_processor(self, dialect, type_): ...

class MSIdentifierPreparer_pymssql(MSIdentifierPreparer):
    def __init__(self, dialect) -> None: ...

class MSDialect_pymssql(MSDialect):
    supports_statement_cache: bool
    supports_native_decimal: bool
    driver: str
    preparer: Any
    colspecs: Any
    @classmethod
    def dbapi(cls): ...
    def create_connect_args(self, url): ...
    def is_disconnect(self, e, connection, cursor): ...
    def set_isolation_level(self, connection, level) -> None: ...

dialect = MSDialect_pymssql



from collections.abc import Mapping
from typing import Any, overload
from typing_extensions import Literal

from ..future.engine import Engine as FutureEngine
from .base import Engine
from .mock import MockConnection
from .url import URL

# Further kwargs are forwarded to the engine, dialect, or pool.
@overload
def create_engine(url: URL | str, *, strategy: Literal["mock"], **kwargs) -> MockConnection: ...  # type: ignore[misc]
@overload
def create_engine(
    url: URL | str, *, module: Any | None = ..., enable_from_linting: bool = ..., future: Literal[True], **kwargs
) -> FutureEngine: ...
@overload
def create_engine(
    url: URL | str, *, module: Any | None = ..., enable_from_linting: bool = ..., future: Literal[False] = ..., **kwargs
) -> Engine: ...
def engine_from_config(configuration: Mapping[str, Any], prefix: str = ..., **kwargs) -> Engine: ...



from datetime import tzinfo
from typing import Any

STDOFFSET: Any
DSTOFFSET: Any
DSTDIFF: Any
ZERO: Any

class _FallbackLocalTimezone(tzinfo):
    def utcoffset(self, dt): ...
    def dst(self, dt): ...
    def tzname(self, dt): ...

def get_localzone(): ...

LOCALTZ: Any



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class ManualLayout(Serialisable):
    tagname: str
    layoutTarget: Any
    xMode: Any
    yMode: Any
    wMode: Any
    hMode: Any
    x: Any
    y: Any
    w: Any
    width: Any
    h: Any
    height: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        layoutTarget: Any | None = ...,
        xMode: Any | None = ...,
        yMode: Any | None = ...,
        wMode: str = ...,
        hMode: str = ...,
        x: Any | None = ...,
        y: Any | None = ...,
        w: Any | None = ...,
        h: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class Layout(Serialisable):
    tagname: str
    manualLayout: Any
    extLst: Any
    __elements__: Any
    def __init__(self, manualLayout: Any | None = ..., extLst: Any | None = ...) -> None: ...



import re
import subprocess
from _typeshed import Incomplete, Self
from collections import OrderedDict
from collections.abc import Callable, Generator
from logging import Logger
from types import TracebackType
from typing import Any

__pkgname__: str
ITEMREX: re.Pattern[str]
SPECREX: re.Pattern[str]
DEVNULL: str
WEEK_ENUM: list[str]
MONTH_ENUM: list[str | None]
SPECIALS: dict[str, str]
SPECIAL_IGNORE: list[str]
S_INFO: list[dict[str, Any]]
PY3: bool
WINOS: bool
POSIX: bool
SYSTEMV: bool
ZERO_PAD: bool
LOG: Logger
CRON_COMMAND: str
SHELL: str
current_user: Callable[[], str | None]

def open_pipe(cmd: str, *args: str, **flags) -> subprocess.Popen[Any]: ...

class CronTab:
    lines: Incomplete
    crons: Incomplete
    filen: Incomplete
    cron_command: Incomplete
    env: Incomplete
    root: bool
    intab: Incomplete
    def __init__(
        self,
        user: bool | str | None = ...,
        tab: Incomplete | None = ...,
        tabfile: Incomplete | None = ...,
        log: Incomplete | None = ...,
    ) -> None: ...
    def __enter__(self: Self) -> Self: ...
    def __exit__(
        self, exc_type: type[BaseException] | None, exc_val: BaseException | None, exc_tb: TracebackType | None
    ) -> None: ...
    @property
    def log(self): ...
    @property
    def user(self) -> str | None: ...
    @property
    def user_opt(self): ...
    def read(self, filename: Incomplete | None = ...) -> None: ...
    def append(self, item, line: str = ..., read: bool = ...) -> None: ...
    def write(self, filename: Incomplete | None = ..., user: Incomplete | None = ..., errors: bool = ...) -> None: ...
    def write_to_user(self, user: bool = ...): ...
    def run_pending(self, **kwargs) -> Generator[Incomplete, None, None]: ...
    def run_scheduler(self, timeout: int = ..., **kwargs) -> Generator[Incomplete, None, None]: ...
    def render(self, errors: bool = ..., specials: bool = ...): ...
    def new(self, command: str = ..., comment: str = ..., user: Incomplete | None = ..., pre_comment: bool = ...) -> CronItem: ...
    def find_command(self, command) -> Generator[Incomplete, None, None]: ...
    def find_comment(self, comment) -> Generator[Incomplete, None, None]: ...
    def find_time(self, *args) -> Generator[Incomplete, None, None]: ...
    @property
    def commands(self) -> Generator[Incomplete, None, None]: ...
    @property
    def comments(self) -> Generator[Incomplete, None, None]: ...
    def remove_all(self, *args, **kwargs): ...
    def remove(self, *items): ...
    def __iter__(self): ...
    def __getitem__(self, i): ...
    def __unicode__(self) -> str: ...
    def __len__(self) -> int: ...

class CronItem:
    cron: Incomplete
    user: Incomplete
    valid: bool
    enabled: bool
    special: bool
    comment: Incomplete
    command: Incomplete
    last_run: Incomplete
    env: Incomplete
    pre_comment: bool
    marker: Incomplete
    stdin: Incomplete
    slices: Incomplete
    def __init__(
        self, command: str = ..., comment: str = ..., user: Incomplete | None = ..., pre_comment: bool = ...
    ) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    @classmethod
    def from_line(cls: type[Self], line: str, user: Incomplete | None = ..., cron: Incomplete | None = ...) -> Self: ...
    def delete(self) -> None: ...
    def set_command(self, cmd: str, parse_stdin: bool = ...) -> None: ...
    def set_comment(self, cmt: str, pre_comment: bool = ...) -> None: ...
    def parse(self, line) -> None: ...
    def enable(self, enabled: bool = ...) -> bool: ...
    def is_enabled(self) -> bool: ...
    def is_valid(self) -> bool: ...
    def render(self, specials: bool = ...) -> str: ...
    def every_reboot(self): ...
    def every(self, unit: int = ...): ...
    def setall(self, *args: Any): ...
    def clear(self): ...
    def frequency(self, year: Incomplete | None = ...): ...
    def frequency_per_year(self, year: Incomplete | None = ...): ...
    def frequency_per_day(self): ...
    def frequency_per_hour(self): ...
    def run_pending(self, now: Incomplete | None = ...): ...
    def run(self): ...
    def schedule(self, date_from: Incomplete | None = ...): ...
    def description(self, **kw: Any): ...
    @property
    def log(self): ...
    @property
    def minute(self): ...
    @property
    def minutes(self): ...
    @property
    def hour(self): ...
    @property
    def hours(self): ...
    @property
    def day(self): ...
    @property
    def dom(self): ...
    @property
    def month(self): ...
    @property
    def months(self): ...
    @property
    def dow(self): ...
    def __len__(self) -> int: ...
    def __getitem__(self, key: str): ...
    def __lt__(self, value): ...
    def __gt__(self, value): ...
    def __unicode__(self) -> str: ...

class Every:
    slices: Incomplete
    unit: Incomplete
    def __init__(self, item, units) -> None: ...
    def set_attr(self, target: int) -> Callable[[], None]: ...
    def year(self) -> None: ...

class CronSlices(list[CronSlice]):
    special: Incomplete
    def __init__(self, *args: Any) -> None: ...
    def is_self_valid(self, *args: Any) -> bool: ...
    @classmethod
    def is_valid(cls, *args: Any) -> bool: ...
    def setall(self, *slices) -> None: ...
    def clean_render(self) -> str: ...
    def render(self, specials: bool = ...) -> str: ...
    def clear(self) -> None: ...
    def frequency(self, year: Incomplete | None = ...): ...
    def frequency_per_year(self, year: Incomplete | None = ...): ...
    def frequency_per_day(self): ...
    def frequency_per_hour(self): ...
    def __eq__(self, arg: object) -> bool: ...

class SundayError(KeyError): ...

class Also:
    obj: Incomplete
    def __init__(self, obj) -> None: ...
    def every(self, *a): ...
    def on(self, *a): ...
    def during(self, *a): ...

class CronSlice:
    min: Incomplete
    max: Incomplete
    name: Incomplete
    enum: Incomplete
    parts: Incomplete
    def __init__(self, info, value: Incomplete | None = ...) -> None: ...
    def __hash__(self) -> int: ...
    def parse(self, value) -> None: ...
    def render(self, resolve: bool = ..., specials: bool = ...): ...
    def __eq__(self, arg: object) -> bool: ...
    def __unicode__(self) -> str: ...
    def every(self, n_value, also: bool = ...): ...
    def on(self, *n_value, **opts): ...
    def during(self, vfrom, vto, also: bool = ...): ...
    @property
    def also(self): ...
    def clear(self) -> None: ...
    def get_range(self, *vrange): ...
    def __iter__(self): ...
    def __len__(self) -> int: ...
    def parse_value(self, val, sunday: Incomplete | None = ...): ...

def get_cronvalue(value, enums): ...

class CronValue:
    text: Incomplete
    value: Incomplete
    def __init__(self, value, enums) -> None: ...
    def __lt__(self, value): ...
    def __int__(self) -> int: ...

class CronRange:
    dangling: Incomplete
    slice: Incomplete
    cron: Incomplete
    seq: int
    def __init__(self, vslice, *vrange) -> None: ...
    vfrom: Incomplete
    vto: Incomplete
    def parse(self, value) -> None: ...
    def all(self) -> None: ...
    def render(self, resolve: bool = ...): ...
    def range(self): ...
    def every(self, value) -> None: ...
    def __lt__(self, value): ...
    def __gt__(self, value): ...
    def __int__(self) -> int: ...
    def __unicode__(self) -> str: ...

class OrderedVariableList(OrderedDict[Incomplete, Incomplete]):
    job: Incomplete
    def __init__(self, *args: Any, **kw: Any) -> None: ...
    @property
    def previous(self): ...
    def all(self): ...
    def __getitem__(self, key): ...



from _typeshed import Incomplete

import _win32typing
from win32.lib.pywintypes import error as error

def ReadEventLog(Handle, Flags, Offset, Size=...) -> list[Incomplete]: ...
def ClearEventLog(handle, eventLogName: str) -> None: ...
def BackupEventLog(handle, eventLogName: str) -> None: ...
def CloseEventLog(handle) -> None: ...
def DeregisterEventSource(handle) -> None: ...
def NotifyChangeEventLog(handle, handle1) -> None: ...
def GetNumberOfEventLogRecords(handle): ...
def GetOldestEventLogRecord(): ...
def OpenEventLog(serverName: str, sourceName: str) -> _win32typing.PyEVTLOG_HANDLE: ...
def RegisterEventSource(serverName: str, sourceName: str): ...
def OpenBackupEventLog(serverName: str, fileName: str) -> _win32typing.PyEVTLOG_HANDLE: ...
def ReportEvent(EventLog: int, Type, Category, EventID, UserSid: _win32typing.PySID, Strings, RawData) -> None: ...
def EvtOpenChannelEnum(Session: _win32typing.PyEVT_HANDLE | None = ..., Flags: int = ...) -> _win32typing.PyEVT_HANDLE: ...
def EvtNextChannelPath(ChannelEnum: _win32typing.PyEVT_HANDLE): ...
def EvtOpenLog(Path, Flags, Session: _win32typing.PyEVT_HANDLE | None = ...) -> _win32typing.PyEVT_HANDLE: ...
def EvtClearLog(
    ChannelPath, TargetFilePath: Incomplete | None = ..., Session: _win32typing.PyEVT_HANDLE | None = ..., Flags: int = ...
) -> None: ...
def EvtExportLog(
    Path, TargetFilePath, Flags, Query: Incomplete | None = ..., Session: _win32typing.PyEVT_HANDLE | None = ...
) -> None: ...
def EvtArchiveExportedLog(LogFilePath, Locale, Session: _win32typing.PyEVT_HANDLE | None = ..., Flags=...) -> None: ...
def EvtGetExtendedStatus(): ...
def EvtQuery(
    Path, Flags, Query: Incomplete | None = ..., Session: _win32typing.PyEVT_HANDLE | None = ...
) -> _win32typing.PyEVT_HANDLE: ...
def EvtNext(
    ResultSet: _win32typing.PyEVT_HANDLE, Count, Timeout: int = ..., Flags=...
) -> tuple[_win32typing.PyEVT_HANDLE, ...]: ...
def EvtSeek(
    ResultSet: _win32typing.PyEVT_HANDLE, Position, Flags, Bookmark: _win32typing.PyEVT_HANDLE | None = ..., Timeout: int = ...
) -> None: ...
def EvtRender(Event: _win32typing.PyEVT_HANDLE, Flags): ...
def EvtSubscribe(
    ChannelPath,
    Flags,
    SignalEvent: Incomplete | None = ...,
    Callback: Incomplete | None = ...,
    Context: Incomplete | None = ...,
    Query: Incomplete | None = ...,
    Session: _win32typing.PyEVT_HANDLE | None = ...,
    Bookmark: _win32typing.PyEVT_HANDLE | None = ...,
) -> _win32typing.PyEVT_HANDLE: ...
def EvtCreateBookmark(BookmarkXML: Incomplete | None = ...) -> _win32typing.PyEVT_HANDLE: ...
def EvtUpdateBookmark(Bookmark: _win32typing.PyEVT_HANDLE, Event: _win32typing.PyEVT_HANDLE) -> _win32typing.PyEVT_HANDLE: ...
def EvtGetChannelConfigProperty(
    ChannelConfig: _win32typing.PyEVT_HANDLE, PropertyId, Flags=...
) -> tuple[Incomplete, Incomplete]: ...
def EvtOpenChannelConfig(
    ChannelPath, Session: _win32typing.PyEVT_HANDLE | None = ..., Flags=...
) -> _win32typing.PyEVT_HANDLE: ...
def EvtOpenSession(
    Login: _win32typing.PyEVT_RPC_LOGIN, LoginClass, Timeout: int = ..., Flags=...
) -> _win32typing.PyEVT_HANDLE: ...
def EvtOpenPublisherEnum(Session: _win32typing.PyEVT_HANDLE | None = ..., Flags: int = ...) -> _win32typing.PyEVT_HANDLE: ...
def EvtNextPublisherId(PublisherEnum: _win32typing.PyEVT_HANDLE): ...
def EvtOpenPublisherMetadata(
    PublisherIdentity,
    Session: _win32typing.PyEVT_HANDLE | None = ...,
    LogFilePath: Incomplete | None = ...,
    Locale: int = ...,
    Flags: int = ...,
) -> _win32typing.PyEVT_HANDLE: ...
def EvtGetPublisherMetadataProperty(
    PublisherMetadata: _win32typing.PyEVT_HANDLE, PropertyId, Flags=...
) -> tuple[Incomplete, Incomplete]: ...
def EvtOpenEventMetadataEnum(PublisherMetadata: _win32typing.PyEVT_HANDLE, Flags=...) -> _win32typing.PyEVT_HANDLE: ...
def EvtNextEventMetadata(EventMetadataEnum: _win32typing.PyEVT_HANDLE, Flags=...) -> _win32typing.PyEVT_HANDLE: ...
def EvtGetEventMetadataProperty(
    EventMetadata: _win32typing.PyEVT_HANDLE, PropertyId, Flags=...
) -> tuple[Incomplete, Incomplete]: ...
def EvtGetLogInfo(Log: _win32typing.PyEVT_HANDLE, PropertyId) -> tuple[Incomplete, Incomplete]: ...
def EvtGetEventInfo(Event: _win32typing.PyEVT_HANDLE, PropertyId) -> tuple[Incomplete, Incomplete]: ...
def EvtGetObjectArraySize(ObjectArray: _win32typing.PyEVT_HANDLE): ...
def EvtGetObjectArrayProperty(
    ObjectArray: _win32typing.PyEVT_HANDLE, PropertyId, ArrayIndex, Flags=...
) -> tuple[Incomplete, Incomplete]: ...
def EvtCreateRenderContext(*args, **kwargs): ...  # incomplete
def EvtFormatMessage(*args, **kwargs): ...  # incomplete

EVENTLOG_AUDIT_FAILURE: int
EVENTLOG_AUDIT_SUCCESS: int
EVENTLOG_BACKWARDS_READ: int
EVENTLOG_END_ALL_PAIRED_EVENTS: int
EVENTLOG_END_PAIRED_EVENT: int
EVENTLOG_ERROR_TYPE: int
EVENTLOG_FORWARDS_READ: int
EVENTLOG_INFORMATION_TYPE: int
EVENTLOG_PAIRED_EVENT_ACTIVE: int
EVENTLOG_PAIRED_EVENT_INACTIVE: int
EVENTLOG_SEEK_READ: int
EVENTLOG_SEQUENTIAL_READ: int
EVENTLOG_START_PAIRED_EVENT: int
EVENTLOG_SUCCESS: int
EVENTLOG_WARNING_TYPE: int
EventMetadataEventChannel: int
EventMetadataEventID: int
EventMetadataEventKeyword: int
EventMetadataEventLevel: int
EventMetadataEventMessageID: int
EventMetadataEventOpcode: int
EventMetadataEventTask: int
EventMetadataEventTemplate: int
EventMetadataEventVersion: int
EvtChannelConfigAccess: int
EvtChannelConfigClassicEventlog: int
EvtChannelConfigEnabled: int
EvtChannelConfigIsolation: int
EvtChannelConfigOwningPublisher: int
EvtChannelConfigPropertyIdEND: int
EvtChannelConfigType: int
EvtChannelLoggingConfigAutoBackup: int
EvtChannelLoggingConfigLogFilePath: int
EvtChannelLoggingConfigMaxSize: int
EvtChannelLoggingConfigRetention: int
EvtChannelPublishingConfigBufferSize: int
EvtChannelPublishingConfigClockType: int
EvtChannelPublishingConfigControlGuid: int
EvtChannelPublishingConfigKeywords: int
EvtChannelPublishingConfigLatency: int
EvtChannelPublishingConfigLevel: int
EvtChannelPublishingConfigMaxBuffers: int
EvtChannelPublishingConfigMinBuffers: int
EvtChannelPublishingConfigSidType: int
EvtEventMetadataPropertyIdEND: int
EvtEventPath: int
EvtEventPropertyIdEND: int
EvtEventQueryIDs: int
EvtExportLogChannelPath: int
EvtExportLogFilePath: int
EvtExportLogTolerateQueryErrors: int
EvtLogAttributes: int
EvtLogCreationTime: int
EvtLogFileSize: int
EvtLogFull: int
EvtLogLastAccessTime: int
EvtLogLastWriteTime: int
EvtLogNumberOfLogRecords: int
EvtLogOldestRecordNumber: int
EvtOpenChannelPath: int
EvtOpenFilePath: int
EvtPublisherMetadataChannelReferenceFlags: int
EvtPublisherMetadataChannelReferenceID: int
EvtPublisherMetadataChannelReferenceIndex: int
EvtPublisherMetadataChannelReferenceMessageID: int
EvtPublisherMetadataChannelReferencePath: int
EvtPublisherMetadataChannelReferences: int
EvtPublisherMetadataHelpLink: int
EvtPublisherMetadataKeywordMessageID: int
EvtPublisherMetadataKeywordName: int
EvtPublisherMetadataKeywords: int
EvtPublisherMetadataKeywordValue: int
EvtPublisherMetadataLevelMessageID: int
EvtPublisherMetadataLevelName: int
EvtPublisherMetadataLevels: int
EvtPublisherMetadataLevelValue: int
EvtPublisherMetadataMessageFilePath: int
EvtPublisherMetadataOpcodeMessageID: int
EvtPublisherMetadataOpcodeName: int
EvtPublisherMetadataOpcodes: int
EvtPublisherMetadataOpcodeValue: int
EvtPublisherMetadataParameterFilePath: int
EvtPublisherMetadataPropertyIdEND: int
EvtPublisherMetadataPublisherGuid: int
EvtPublisherMetadataPublisherMessageID: int
EvtPublisherMetadataResourceFilePath: int
EvtPublisherMetadataTaskEventGuid: int
EvtPublisherMetadataTaskMessageID: int
EvtPublisherMetadataTaskName: int
EvtPublisherMetadataTasks: int
EvtPublisherMetadataTaskValue: int
EvtQueryChannelPath: int
EvtQueryFilePath: int
EvtQueryForwardDirection: int
EvtQueryReverseDirection: int
EvtQueryTolerateQueryErrors: int
EvtRenderBookmark: int
EvtRenderEventValues: int
EvtRenderEventXml: int
EvtRpcLogin: int
EvtRpcLoginAuthDefault: int
EvtRpcLoginAuthKerberos: int
EvtRpcLoginAuthNegotiate: int
EvtRpcLoginAuthNTLM: int
EvtSeekOriginMask: int
EvtSeekRelativeToBookmark: int
EvtSeekRelativeToCurrent: int
EvtSeekRelativeToFirst: int
EvtSeekRelativeToLast: int
EvtSeekStrict: int
EvtSubscribeActionDeliver: int
EvtSubscribeActionError: int
EvtSubscribeOriginMask: int
EvtSubscribeStartAfterBookmark: int
EvtSubscribeStartAtOldestRecord: int
EvtSubscribeStrict: int
EvtSubscribeToFutureEvents: int
EvtSubscribeTolerateQueryErrors: int
EvtVarTypeAnsiString: int
EvtVarTypeBinary: int
EvtVarTypeBoolean: int
EvtVarTypeByte: int
EvtVarTypeDouble: int
EvtVarTypeEvtHandle: int
EvtVarTypeEvtXml: int
EvtVarTypeFileTime: int
EvtVarTypeGuid: int
EvtVarTypeHexInt32: int
EvtVarTypeHexInt64: int
EvtVarTypeInt16: int
EvtVarTypeInt32: int
EvtVarTypeInt64: int
EvtVarTypeNull: int
EvtVarTypeSByte: int
EvtVarTypeSid: int
EvtVarTypeSingle: int
EvtVarTypeSizeT: int
EvtVarTypeString: int
EvtVarTypeSysTime: int
EvtVarTypeUInt16: int
EvtVarTypeUInt32: int
EvtVarTypeUInt64: int
EvtChannelPublisherList: int
EvtFormatMessageChannel: int
EvtFormatMessageEvent: int
EvtFormatMessageId: int
EvtFormatMessageKeyword: int
EvtFormatMessageLevel: int
EvtFormatMessageOpcode: int
EvtFormatMessageProvider: int
EvtFormatMessageTask: int
EvtFormatMessageXml: int
EvtRenderContextSystem: int
EvtRenderContextUser: int
EvtRenderContextValues: int
EvtSystemActivityID: int
EvtSystemChannel: int
EvtSystemComputer: int
EvtSystemEventID: int
EvtSystemEventRecordId: int
EvtSystemKeywords: int
EvtSystemLevel: int
EvtSystemOpcode: int
EvtSystemProcessID: int
EvtSystemPropertyIdEND: int
EvtSystemProviderGuid: int
EvtSystemProviderName: int
EvtSystemQualifiers: int
EvtSystemRelatedActivityID: int
EvtSystemTask: int
EvtSystemThreadID: int
EvtSystemTimeCreated: int
EvtSystemUserID: int
EvtSystemVersion: int
UNICODE: int



from typing import Any

unix_socket_available: bool
SESSION_TERMINATED_BY_SERVER: str
TRANSACTION_ERROR: str
RESPONSE_COMPLETE: str

class BaseStrategy:
    connection: Any
    sync: Any
    no_real_dsa: Any
    pooled: Any
    can_stream: Any
    referral_cache: Any
    thread_safe: bool
    def __init__(self, ldap_connection) -> None: ...
    def open(self, reset_usage: bool = ..., read_server_info: bool = ...) -> None: ...
    def close(self) -> None: ...
    def send(self, message_type, request, controls: Any | None = ...): ...
    def get_response(self, message_id, timeout: Any | None = ..., get_request: bool = ...): ...
    @staticmethod
    def compute_ldap_message_size(data): ...
    def decode_response(self, ldap_message): ...
    def decode_response_fast(self, ldap_message): ...
    @staticmethod
    def decode_control(control): ...
    @staticmethod
    def decode_control_fast(control, from_server: bool = ...): ...
    @staticmethod
    def decode_request(message_type, component, controls: Any | None = ...): ...
    def valid_referral_list(self, referrals): ...
    def do_next_range_search(self, request, response, attr_name): ...
    def do_search_on_auto_range(self, request, response): ...
    def create_referral_connection(self, referrals): ...
    def do_operation_on_referral(self, request, referrals): ...
    def sending(self, ldap_message) -> None: ...
    def receiving(self) -> None: ...
    def post_send_single_response(self, message_id) -> None: ...
    def post_send_search(self, message_id) -> None: ...
    def get_stream(self) -> None: ...
    def set_stream(self, value) -> None: ...
    def unbind_referral_cache(self) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Font(Serialisable):
    UNDERLINE_DOUBLE: str
    UNDERLINE_DOUBLE_ACCOUNTING: str
    UNDERLINE_SINGLE: str
    UNDERLINE_SINGLE_ACCOUNTING: str
    name: Any
    charset: Any
    family: Any
    sz: Any
    size: Any
    b: Any
    bold: Any
    i: Any
    italic: Any
    strike: Any
    strikethrough: Any
    outline: Any
    shadow: Any
    condense: Any
    extend: Any
    u: Any
    underline: Any
    vertAlign: Any
    color: Any
    scheme: Any
    tagname: str
    __elements__: Any
    def __init__(
        self,
        name: Any | None = ...,
        sz: Any | None = ...,
        b: Any | None = ...,
        i: Any | None = ...,
        charset: Any | None = ...,
        u: Any | None = ...,
        strike: Any | None = ...,
        color: Any | None = ...,
        scheme: Any | None = ...,
        family: Any | None = ...,
        size: Any | None = ...,
        bold: Any | None = ...,
        italic: Any | None = ...,
        strikethrough: Any | None = ...,
        underline: Any | None = ...,
        vertAlign: Any | None = ...,
        outline: Any | None = ...,
        shadow: Any | None = ...,
        condense: Any | None = ...,
        extend: Any | None = ...,
    ) -> None: ...
    @classmethod
    def from_tree(cls, node): ...

DEFAULT_FONT: Any



from typing import Any
from typing_extensions import TypeAlias

from . import util

_AssignmentStmt: TypeAlias = Any  # mypy.nodes.AssignmentStmt
_NameExpr: TypeAlias = Any  # mypy.nodes.NameExpr
_StrExpr: TypeAlias = Any  # mypy.nodes.StrExpr
_SemanticAnalyzerPluginInterface: TypeAlias = Any  # mypy.plugin.SemanticAnalyzerPluginInterface
_ProperType: TypeAlias = Any  # mypy.types.ProperType

def apply_mypy_mapped_attr(
    cls, api: _SemanticAnalyzerPluginInterface, item: _NameExpr | _StrExpr, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...
def re_apply_declarative_assignments(
    cls, api: _SemanticAnalyzerPluginInterface, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...
def apply_type_to_mapped_statement(
    api: _SemanticAnalyzerPluginInterface,
    stmt: _AssignmentStmt,
    lvalue: _NameExpr,
    left_hand_explicit_type: _ProperType | None,
    python_type_for_type: _ProperType | None,
) -> None: ...
def add_additional_orm_attributes(
    cls, api: _SemanticAnalyzerPluginInterface, attributes: list[util.SQLAlchemyAttribute]
) -> None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class ChartContainer(Serialisable):
    tagname: str
    title: Any
    autoTitleDeleted: Any
    pivotFmts: Any
    view3D: Any
    floor: Any
    sideWall: Any
    backWall: Any
    plotArea: Any
    legend: Any
    plotVisOnly: Any
    dispBlanksAs: Any
    showDLblsOverMax: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        title: Any | None = ...,
        autoTitleDeleted: Any | None = ...,
        pivotFmts=...,
        view3D: Any | None = ...,
        floor: Any | None = ...,
        sideWall: Any | None = ...,
        backWall: Any | None = ...,
        plotArea: Any | None = ...,
        legend: Any | None = ...,
        plotVisOnly: bool = ...,
        dispBlanksAs: str = ...,
        showDLblsOverMax: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class Protection(Serialisable):
    tagname: str
    chartObject: Any
    data: Any
    formatting: Any
    selection: Any
    userInterface: Any
    __elements__: Any
    def __init__(
        self,
        chartObject: Any | None = ...,
        data: Any | None = ...,
        formatting: Any | None = ...,
        selection: Any | None = ...,
        userInterface: Any | None = ...,
    ) -> None: ...

class ExternalData(Serialisable):
    tagname: str
    autoUpdate: Any
    id: Any
    def __init__(self, autoUpdate: Any | None = ..., id: Any | None = ...) -> None: ...

class ChartSpace(Serialisable):
    tagname: str
    date1904: Any
    lang: Any
    roundedCorners: Any
    style: Any
    clrMapOvr: Any
    pivotSource: Any
    protection: Any
    chart: Any
    spPr: Any
    graphicalProperties: Any
    txPr: Any
    textProperties: Any
    externalData: Any
    printSettings: Any
    userShapes: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        date1904: Any | None = ...,
        lang: Any | None = ...,
        roundedCorners: Any | None = ...,
        style: Any | None = ...,
        clrMapOvr: Any | None = ...,
        pivotSource: Any | None = ...,
        protection: Any | None = ...,
        chart: Any | None = ...,
        spPr: Any | None = ...,
        txPr: Any | None = ...,
        externalData: Any | None = ...,
        printSettings: Any | None = ...,
        userShapes: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...
    def to_tree(self, tagname: Any | None = ..., idx: Any | None = ..., namespace: Any | None = ...): ...



from _typeshed import Self
from collections.abc import Iterator
from typing import Any, Generic, TypeVar
from typing_extensions import Literal, TypeAlias

from ..sql.annotation import SupportsCloneAnnotations
from ..sql.base import Executable
from ..sql.selectable import GroupedElement, HasHints, HasPrefixes, HasSuffixes, SelectBase, _SelectFromElements
from . import interfaces
from .context import QueryContext as QueryContext
from .util import aliased as aliased

__all__ = ["Query", "QueryContext", "aliased"]

_T = TypeVar("_T")
_SynchronizeSessionArgument: TypeAlias = Literal[False, "evaluate", "fetch"]

class Query(_SelectFromElements, SupportsCloneAnnotations, HasPrefixes, HasSuffixes, HasHints, Executable, Generic[_T]):
    logger: Any
    load_options: Any
    session: Any
    def __init__(self, entities, session: Any | None = ...) -> None: ...
    @property
    def statement(self): ...
    def subquery(self, name: str | None = ..., with_labels: bool = ..., reduce_columns: bool = ...): ...
    def cte(self, name: Any | None = ..., recursive: bool = ..., nesting: bool = ...): ...
    def label(self, name): ...
    def as_scalar(self): ...
    def scalar_subquery(self): ...
    @property
    def selectable(self): ...
    def __clause_element__(self): ...
    def only_return_tuples(self: Self, value) -> Self: ...
    @property
    def is_single_entity(self): ...
    def enable_eagerloads(self: Self, value) -> Self: ...
    def with_labels(self): ...
    apply_labels: Any
    @property
    def get_label_style(self): ...
    def set_label_style(self, style): ...
    def enable_assertions(self: Self, value) -> Self: ...
    @property
    def whereclause(self): ...
    def with_polymorphic(self: Self, cls_or_mappers, selectable: Any | None = ..., polymorphic_on: Any | None = ...) -> Self: ...
    def yield_per(self: Self, count) -> Self: ...
    def get(self, ident): ...
    @property
    def lazy_loaded_from(self): ...
    def correlate(self: Self, *fromclauses) -> Self: ...
    def autoflush(self: Self, setting) -> Self: ...
    def populate_existing(self: Self) -> Self: ...
    def with_parent(self, instance, property: Any | None = ..., from_entity: Any | None = ...): ...
    def add_entity(self: Self, entity, alias: Any | None = ...) -> Self: ...
    def with_session(self: Self, session) -> Self: ...
    def from_self(self, *entities): ...
    def values(self, *columns): ...
    def value(self, column): ...
    def with_entities(self: Self, *entities) -> Self: ...
    def add_columns(self: Self, *column) -> Self: ...
    def add_column(self, column): ...
    def options(self: Self, *args) -> Self: ...
    def with_transformation(self, fn): ...
    def get_execution_options(self): ...
    def execution_options(self: Self, **kwargs) -> Self: ...
    def with_for_update(
        self: Self, read: bool = ..., nowait: bool = ..., of: Any | None = ..., skip_locked: bool = ..., key_share: bool = ...
    ) -> Self: ...
    def params(self: Self, *args, **kwargs) -> Self: ...
    def where(self, *criterion): ...
    def filter(self: Self, *criterion) -> Self: ...
    def filter_by(self: Self, **kwargs) -> Self: ...
    def order_by(self: Self, *clauses) -> Self: ...
    def group_by(self: Self, *clauses) -> Self: ...
    def having(self: Self, criterion) -> Self: ...
    def union(self, *q): ...
    def union_all(self, *q): ...
    def intersect(self, *q): ...
    def intersect_all(self, *q): ...
    def except_(self, *q): ...
    def except_all(self, *q): ...
    def join(self: Self, target, *props, **kwargs) -> Self: ...
    def outerjoin(self: Self, target, *props, **kwargs) -> Self: ...
    def reset_joinpoint(self: Self) -> Self: ...
    def select_from(self: Self, *from_obj) -> Self: ...
    def select_entity_from(self: Self, from_obj) -> Self: ...
    def __getitem__(self, item): ...
    def slice(self: Self, start, stop) -> Self: ...
    def limit(self: Self, limit) -> Self: ...
    def offset(self: Self, offset) -> Self: ...
    def distinct(self: Self, *expr) -> Self: ...
    def all(self) -> list[_T]: ...
    def from_statement(self: Self, statement) -> Self: ...
    def first(self) -> _T | None: ...
    def one_or_none(self): ...
    def one(self): ...
    def scalar(self) -> Any: ...  # type: ignore[override]
    def __iter__(self) -> Iterator[_T]: ...
    @property
    def column_descriptions(self): ...
    def instances(self, result_proxy, context: Any | None = ...): ...
    def merge_result(self, iterator, load: bool = ...): ...
    def exists(self): ...
    def count(self) -> int: ...
    def delete(self, synchronize_session: _SynchronizeSessionArgument = ...) -> int: ...
    def update(self, values, synchronize_session: _SynchronizeSessionArgument = ..., update_args: Any | None = ...): ...

class FromStatement(GroupedElement, SelectBase, Executable):
    __visit_name__: str
    element: Any
    def __init__(self, entities, element) -> None: ...
    def get_label_style(self): ...
    def set_label_style(self, label_style): ...
    def get_children(self, **kw) -> None: ...  # type: ignore[override]

class AliasOption(interfaces.LoaderOption):
    def __init__(self, alias) -> None: ...
    inherit_cache: bool
    def process_compile_state(self, compile_state) -> None: ...

class BulkUD:
    query: Any
    mapper: Any
    def __init__(self, query) -> None: ...
    @property
    def session(self): ...

class BulkUpdate(BulkUD):
    values: Any
    update_kwargs: Any
    def __init__(self, query, values, update_kwargs) -> None: ...

class BulkDelete(BulkUD): ...



#!/usr/bin/env python3

import os
import shutil
import subprocess
import sys
from pathlib import Path

_PYRIGHT_VERSION = "1.1.278"  # Must match .github/workflows/tests.yml.
_WELL_KNOWN_FILE = Path("tests", "pyright_test.py")


def main() -> None:
    if not _WELL_KNOWN_FILE.exists():
        print("pyright_test.py must be run from the typeshed root directory", file=sys.stderr)
        sys.exit(1)

    # subprocess.run on Windows does not look in PATH.
    npx = shutil.which("npx")

    if npx is None:
        print("error finding npx; is Node.js installed?", file=sys.stderr)
        sys.exit(1)

    try:
        subprocess.run([npx, "--version"])
    except OSError:
        print("error running npx; is Node.js installed?", file=sys.stderr)
        sys.exit(1)

    os.environ["PYRIGHT_PYTHON_FORCE_VERSION"] = _PYRIGHT_VERSION
    command = [npx, f"pyright@{_PYRIGHT_VERSION}"] + sys.argv[1:]
    print("Running:", " ".join(command))

    ret = subprocess.run(command).returncode
    sys.exit(ret)


if __name__ == "__main__":
    main()



from typing import Any

from openpyxl.descriptors import Sequence
from openpyxl.descriptors.serialisable import Serialisable

FILL_NONE: str
FILL_SOLID: str
FILL_PATTERN_DARKDOWN: str
FILL_PATTERN_DARKGRAY: str
FILL_PATTERN_DARKGRID: str
FILL_PATTERN_DARKHORIZONTAL: str
FILL_PATTERN_DARKTRELLIS: str
FILL_PATTERN_DARKUP: str
FILL_PATTERN_DARKVERTICAL: str
FILL_PATTERN_GRAY0625: str
FILL_PATTERN_GRAY125: str
FILL_PATTERN_LIGHTDOWN: str
FILL_PATTERN_LIGHTGRAY: str
FILL_PATTERN_LIGHTGRID: str
FILL_PATTERN_LIGHTHORIZONTAL: str
FILL_PATTERN_LIGHTTRELLIS: str
FILL_PATTERN_LIGHTUP: str
FILL_PATTERN_LIGHTVERTICAL: str
FILL_PATTERN_MEDIUMGRAY: str
fills: Any

class Fill(Serialisable):
    tagname: str
    @classmethod
    def from_tree(cls, el): ...

class PatternFill(Fill):
    tagname: str
    __elements__: Any
    patternType: Any
    fill_type: Any
    fgColor: Any
    start_color: Any
    bgColor: Any
    end_color: Any
    def __init__(
        self,
        patternType: Any | None = ...,
        fgColor=...,
        bgColor=...,
        fill_type: Any | None = ...,
        start_color: Any | None = ...,
        end_color: Any | None = ...,
    ) -> None: ...
    def to_tree(self, tagname: Any | None = ..., idx: Any | None = ...): ...  # type: ignore[override]

DEFAULT_EMPTY_FILL: Any
DEFAULT_GRAY_FILL: Any

class Stop(Serialisable):
    tagname: str
    position: Any
    color: Any
    def __init__(self, color, position) -> None: ...

class StopList(Sequence):
    expected_type: Any
    def __set__(self, obj, values) -> None: ...

class GradientFill(Fill):
    tagname: str
    type: Any
    fill_type: Any
    degree: Any
    left: Any
    right: Any
    top: Any
    bottom: Any
    stop: Any
    def __init__(
        self, type: str = ..., degree: int = ..., left: int = ..., right: int = ..., top: int = ..., bottom: int = ..., stop=...
    ) -> None: ...
    def __iter__(self): ...
    def to_tree(self, tagname: Any | None = ..., namespace: Any | None = ..., idx: Any | None = ...): ...  # type: ignore[override]



from abc import abstractmethod
from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

from ._3d import _3DBase
from ._chart import ChartBase

class BandFormat(Serialisable):
    tagname: str
    idx: Any
    spPr: Any
    graphicalProperties: Any
    __elements__: Any
    def __init__(self, idx: int = ..., spPr: Any | None = ...) -> None: ...

class BandFormatList(Serialisable):
    tagname: str
    bandFmt: Any
    __elements__: Any
    def __init__(self, bandFmt=...) -> None: ...

class _SurfaceChartBase(ChartBase):
    wireframe: Any
    ser: Any
    bandFmts: Any
    __elements__: Any
    def __init__(self, wireframe: Any | None = ..., ser=..., bandFmts: Any | None = ..., **kw) -> None: ...
    @property
    @abstractmethod
    def tagname(self) -> str: ...

class SurfaceChart3D(_SurfaceChartBase, _3DBase):
    tagname: str
    wireframe: Any
    ser: Any
    bandFmts: Any
    extLst: Any
    x_axis: Any
    y_axis: Any
    z_axis: Any
    __elements__: Any
    def __init__(self, **kw) -> None: ...

class SurfaceChart(SurfaceChart3D):
    tagname: str
    wireframe: Any
    ser: Any
    bandFmts: Any
    extLst: Any
    __elements__: Any
    def __init__(self, **kw) -> None: ...



from typing import Any

from braintree.address import Address as Address
from braintree.credit_card_verification import CreditCardVerification as CreditCardVerification
from braintree.resource import Resource as Resource

class VisaCheckoutCard(Resource):
    billing_address: Any
    subscriptions: Any
    verification: Any
    def __init__(self, gateway, attributes): ...
    @property
    def expiration_date(self): ...
    @property
    def masked_number(self): ...



from typing import Any, TypeVar

from pygments.formatter import Formatter

_T = TypeVar("_T", str, bytes)

class HtmlFormatter(Formatter[_T]):
    name: str
    aliases: Any
    filenames: Any
    title: Any
    nowrap: Any
    noclasses: Any
    classprefix: Any
    cssclass: Any
    cssstyles: Any
    prestyles: Any
    cssfile: Any
    noclobber_cssfile: Any
    tagsfile: Any
    tagurlformat: Any
    filename: Any
    wrapcode: Any
    span_element_openers: Any
    linenos: int
    linenostart: Any
    linenostep: Any
    linenospecial: Any
    nobackground: Any
    lineseparator: Any
    lineanchors: Any
    linespans: Any
    anchorlinenos: Any
    hl_lines: Any
    def get_style_defs(self, arg: Any | None = ...): ...
    def get_token_style_defs(self, arg: Any | None = ...): ...
    def get_background_style_defs(self, arg: Any | None = ...): ...
    def get_linenos_style_defs(self): ...
    def get_css_prefix(self, arg): ...
    def wrap(self, source): ...
    def format_unencoded(self, tokensource, outfile) -> None: ...



from typing import Any

from sqlalchemy.sql import ClauseElement

from ...engine import default
from ...sql import compiler, sqltypes
from ...types import (
    BLOB as BLOB,
    CHAR as CHAR,
    CLOB as CLOB,
    FLOAT as FLOAT,
    INTEGER as INTEGER,
    NCHAR as NCHAR,
    NVARCHAR as NVARCHAR,
    TIMESTAMP as TIMESTAMP,
    VARCHAR as VARCHAR,
)

RESERVED_WORDS: Any
NO_ARG_FNS: Any

class RAW(sqltypes._Binary):
    __visit_name__: str

OracleRaw = RAW

class NCLOB(sqltypes.Text):
    __visit_name__: str

class VARCHAR2(VARCHAR):
    __visit_name__: str

NVARCHAR2 = NVARCHAR

class NUMBER(sqltypes.Numeric, sqltypes.Integer):
    __visit_name__: str
    def __init__(self, precision: Any | None = ..., scale: Any | None = ..., asdecimal: Any | None = ...) -> None: ...
    def adapt(self, impltype): ...

class DOUBLE_PRECISION(sqltypes.Float):
    __visit_name__: str

class BINARY_DOUBLE(sqltypes.Float):
    __visit_name__: str

class BINARY_FLOAT(sqltypes.Float):
    __visit_name__: str

class BFILE(sqltypes.LargeBinary):
    __visit_name__: str

class LONG(sqltypes.Text):
    __visit_name__: str

class DATE(sqltypes.DateTime):
    __visit_name__: str

class INTERVAL(sqltypes.NativeForEmulated, sqltypes._AbstractInterval):
    __visit_name__: str
    day_precision: Any
    second_precision: Any
    def __init__(self, day_precision: Any | None = ..., second_precision: Any | None = ...) -> None: ...
    def as_generic(self, allow_nulltype: bool = ...): ...
    def coerce_compared_value(self, op, value): ...

class ROWID(sqltypes.TypeEngine):
    __visit_name__: str

class _OracleBoolean(sqltypes.Boolean):
    def get_dbapi_type(self, dbapi): ...

colspecs: Any
ischema_names: Any

class OracleTypeCompiler(compiler.GenericTypeCompiler):
    def visit_datetime(self, type_, **kw): ...
    def visit_float(self, type_, **kw): ...
    def visit_unicode(self, type_, **kw): ...
    def visit_INTERVAL(self, type_, **kw): ...
    def visit_LONG(self, type_, **kw): ...
    def visit_TIMESTAMP(self, type_, **kw): ...
    def visit_DOUBLE_PRECISION(self, type_, **kw): ...
    def visit_BINARY_DOUBLE(self, type_, **kw): ...
    def visit_BINARY_FLOAT(self, type_, **kw): ...
    def visit_FLOAT(self, type_, **kw): ...
    def visit_NUMBER(self, type_, **kw): ...
    def visit_string(self, type_, **kw): ...
    def visit_VARCHAR2(self, type_, **kw): ...
    def visit_NVARCHAR2(self, type_, **kw): ...
    visit_NVARCHAR: Any
    def visit_VARCHAR(self, type_, **kw): ...
    def visit_text(self, type_, **kw): ...
    def visit_unicode_text(self, type_, **kw): ...
    def visit_large_binary(self, type_, **kw): ...
    def visit_big_integer(self, type_, **kw): ...
    def visit_boolean(self, type_, **kw): ...
    def visit_RAW(self, type_, **kw): ...
    def visit_ROWID(self, type_, **kw): ...

class OracleCompiler(compiler.SQLCompiler):
    compound_keywords: Any
    def __init__(self, *args, **kwargs) -> None: ...
    def visit_mod_binary(self, binary, operator, **kw): ...
    def visit_now_func(self, fn, **kw): ...
    def visit_char_length_func(self, fn, **kw): ...
    def visit_match_op_binary(self, binary, operator, **kw): ...
    def visit_true(self, expr, **kw): ...
    def visit_false(self, expr, **kw): ...
    def get_cte_preamble(self, recursive): ...
    def get_select_hint_text(self, byfroms): ...
    def function_argspec(self, fn, **kw): ...
    def visit_function(self, func, **kw): ...
    def visit_table_valued_column(self, element, **kw): ...
    def default_from(self): ...
    def visit_join(self, join, from_linter: Any | None = ..., **kwargs): ...  # type: ignore[override]
    def visit_outer_join_column(self, vc, **kw): ...
    def visit_sequence(self, seq, **kw): ...
    def get_render_as_alias_suffix(self, alias_name_text): ...
    has_out_parameters: bool
    def returning_clause(self, stmt, returning_cols): ...
    def translate_select_structure(self, select_stmt, **kwargs): ...
    def limit_clause(self, select, **kw): ...
    def visit_empty_set_expr(self, type_): ...
    def for_update_clause(self, select, **kw): ...
    def visit_is_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_is_not_distinct_from_binary(self, binary, operator, **kw): ...
    def visit_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_not_regexp_match_op_binary(self, binary, operator, **kw): ...
    def visit_regexp_replace_op_binary(self, binary, operator, **kw): ...

class OracleDDLCompiler(compiler.DDLCompiler):
    def define_constraint_cascades(self, constraint): ...
    def visit_drop_table_comment(self, drop): ...
    def visit_create_index(self, create): ...
    def post_create_table(self, table): ...
    def get_identity_options(self, identity_options): ...
    def visit_computed_column(self, generated): ...
    def visit_identity_column(self, identity, **kw): ...

class OracleIdentifierPreparer(compiler.IdentifierPreparer):
    reserved_words: Any
    illegal_initial_characters: Any
    def format_savepoint(self, savepoint): ...

class OracleExecutionContext(default.DefaultExecutionContext):
    def fire_sequence(self, seq, type_): ...

class OracleDialect(default.DefaultDialect):
    name: str
    supports_statement_cache: bool
    supports_alter: bool
    supports_unicode_statements: bool
    supports_unicode_binds: bool
    max_identifier_length: int
    supports_simple_order_by_label: bool
    cte_follows_insert: bool
    supports_sequences: bool
    sequences_optional: bool
    postfetch_lastrowid: bool
    default_paramstyle: str
    colspecs: Any
    ischema_names: Any
    requires_name_normalize: bool
    supports_comments: bool
    supports_default_values: bool
    supports_default_metavalue: bool
    supports_empty_insert: bool
    supports_identity_columns: bool
    statement_compiler: Any
    ddl_compiler: Any
    type_compiler: Any
    preparer: Any
    reflection_options: Any
    construct_arguments: Any
    use_ansi: Any
    optimize_limits: Any
    exclude_tablespaces: Any
    def __init__(
        self,
        use_ansi: bool = ...,
        optimize_limits: bool = ...,
        use_binds_for_limits: Any | None = ...,
        use_nchar_for_unicode: bool = ...,
        exclude_tablespaces=...,
        **kwargs,
    ) -> None: ...
    implicit_returning: Any
    def initialize(self, connection) -> None: ...
    def do_release_savepoint(self, connection, name) -> None: ...
    def get_isolation_level(self, connection) -> None: ...
    def get_default_isolation_level(self, dbapi_conn): ...
    def set_isolation_level(self, connection, level) -> None: ...
    def has_table(self, connection, table_name, schema: Any | None = ...): ...  # type: ignore[override]
    def has_sequence(self, connection, sequence_name, schema: Any | None = ...): ...  # type: ignore[override]
    def get_schema_names(self, connection, **kw): ...
    def get_table_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_temp_table_names(self, connection, **kw): ...
    def get_view_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_sequence_names(self, connection, schema: Any | None = ..., **kw): ...
    def get_table_options(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_columns(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_table_comment(
        self, connection, table_name, schema: Any | None = ..., resolve_synonyms: bool = ..., dblink: str = ..., **kw
    ): ...
    def get_indexes(
        self, connection, table_name, schema: Any | None = ..., resolve_synonyms: bool = ..., dblink: str = ..., **kw
    ): ...
    def get_pk_constraint(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_foreign_keys(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_unique_constraints(self, connection, table_name, schema: Any | None = ..., **kw): ...
    def get_view_definition(
        self, connection, view_name, schema: Any | None = ..., resolve_synonyms: bool = ..., dblink: str = ..., **kw
    ): ...
    def get_check_constraints(self, connection, table_name, schema: Any | None = ..., include_all: bool = ..., **kw): ...

class _OuterJoinColumn(ClauseElement):
    __visit_name__: str
    column: Any
    def __init__(self, column) -> None: ...



from _typeshed import SupportsRead
from collections.abc import Mapping, Sequence
from typing import IO, Any
from typing_extensions import TypeAlias

from ._yaml import CEmitter, CParser
from .constructor import BaseConstructor, FullConstructor, SafeConstructor, UnsafeConstructor
from .representer import BaseRepresenter, SafeRepresenter
from .resolver import BaseResolver, Resolver

__all__ = ["CBaseLoader", "CSafeLoader", "CFullLoader", "CUnsafeLoader", "CLoader", "CBaseDumper", "CSafeDumper", "CDumper"]

_Readable: TypeAlias = SupportsRead[str | bytes]

class CBaseLoader(CParser, BaseConstructor, BaseResolver):
    def __init__(self, stream: str | bytes | _Readable) -> None: ...

class CLoader(CParser, SafeConstructor, Resolver):
    def __init__(self, stream: str | bytes | _Readable) -> None: ...

class CSafeLoader(CParser, SafeConstructor, Resolver):
    def __init__(self, stream: str | bytes | _Readable) -> None: ...

class CFullLoader(CParser, FullConstructor, Resolver):
    def __init__(self, stream: str | bytes | _Readable) -> None: ...

class CUnsafeLoader(CParser, UnsafeConstructor, Resolver):
    def __init__(self, stream: str | bytes | _Readable) -> None: ...

class CBaseDumper(CEmitter, BaseRepresenter, BaseResolver):
    def __init__(
        self,
        stream: IO[Any],
        default_style: str | None = ...,
        default_flow_style: bool | None = ...,
        canonical: Any | None = ...,
        indent: int | None = ...,
        width: int | None = ...,
        allow_unicode: Any | None = ...,
        line_break: str | None = ...,
        encoding: str | None = ...,
        explicit_start: Any | None = ...,
        explicit_end: Any | None = ...,
        version: Sequence[int] | None = ...,
        tags: Mapping[str, str] | None = ...,
        sort_keys: bool = ...,
    ) -> None: ...

class CDumper(CEmitter, SafeRepresenter, Resolver):
    def __init__(
        self,
        stream: IO[Any],
        default_style: str | None = ...,
        default_flow_style: bool = ...,
        canonical: Any | None = ...,
        indent: int | None = ...,
        width: int | None = ...,
        allow_unicode: Any | None = ...,
        line_break: str | None = ...,
        encoding: str | None = ...,
        explicit_start: Any | None = ...,
        explicit_end: Any | None = ...,
        version: Sequence[int] | None = ...,
        tags: Mapping[str, str] | None = ...,
        sort_keys: bool = ...,
    ) -> None: ...

CSafeDumper = CDumper



from collections import deque
from collections.abc import Iterable

from d3dshot.capture_output import CaptureOutput as CaptureOutput, CaptureOutputs as CaptureOutputs, _Frame
from d3dshot.display import Display as Display

class Singleton(type): ...

class D3DShot(metaclass=Singleton):
    displays: list[Display]
    display: Display
    capture_output: CaptureOutput
    frame_buffer_size: int
    frame_buffer: deque[_Frame]
    previous_screenshot: _Frame | None
    region: tuple[int, int, int, int] | None

    def __init__(
        self,
        capture_output: CaptureOutputs = ...,
        frame_buffer_size: int = ...,
        pil_is_available: bool = ...,
        numpy_is_available: bool = ...,
        pytorch_is_available: bool = ...,
        pytorch_gpu_is_available: bool = ...,
    ) -> None: ...
    @property
    def is_capturing(self) -> bool: ...
    def get_latest_frame(self) -> _Frame | None: ...
    def get_frame(self, frame_index: int) -> _Frame | None: ...
    def get_frames(self, frame_indices: Iterable[int]) -> list[_Frame]: ...
    def get_frame_stack(self, frame_indices: Iterable[int], stack_dimension: str | None = ...) -> _Frame: ...
    def screenshot(self, region: tuple[int, int, int, int] | None = ...) -> _Frame | None: ...
    def screenshot_to_disk(
        self, directory: str | None = ..., file_name: str | None = ..., region: tuple[int, int, int, int] | None = ...
    ) -> str: ...
    def frame_buffer_to_disk(self, directory: str | None = ...) -> None: ...
    def capture(self, target_fps: int = ..., region: tuple[int, int, int, int] | None = ...) -> bool: ...
    def screenshot_every(self, interval: float, region: tuple[int, int, int, int] | None = ...) -> bool: ...
    def screenshot_to_disk_every(
        self, interval: float, directory: str | None = ..., region: tuple[int, int, int, int] | None = ...
    ) -> bool: ...
    def stop(self) -> bool: ...
    def benchmark(self) -> None: ...
    def detect_displays(self) -> None: ...



from _typeshed import (
    OpenBinaryMode,
    OpenBinaryModeReading,
    OpenBinaryModeUpdating,
    OpenBinaryModeWriting,
    OpenTextMode,
    StrOrBytesPath,
)
from asyncio import AbstractEventLoop
from collections.abc import Callable
from typing import Any, overload
from typing_extensions import Literal, TypeAlias

from ..base import AiofilesContextManager
from .binary import AsyncBufferedIOBase, AsyncBufferedReader, AsyncFileIO, _UnknownAsyncBinaryIO
from .text import AsyncTextIOWrapper

_OpenFile: TypeAlias = StrOrBytesPath | int
_Opener: TypeAlias = Callable[[str, int], int]

# Text mode: always returns AsyncTextIOWrapper
@overload
def open(
    file: _OpenFile,
    mode: OpenTextMode = ...,
    buffering: int = ...,
    encoding: str | None = ...,
    errors: str | None = ...,
    newline: str | None = ...,
    closefd: bool = ...,
    opener: _Opener | None = ...,
    *,
    loop: AbstractEventLoop | None = ...,
    executor: Any | None = ...,
) -> AiofilesContextManager[None, None, AsyncTextIOWrapper]: ...

# Unbuffered binary: returns a FileIO
@overload
def open(
    file: _OpenFile,
    mode: OpenBinaryMode,
    buffering: Literal[0],
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
    closefd: bool = ...,
    opener: _Opener | None = ...,
    *,
    loop: AbstractEventLoop | None = ...,
    executor: Any | None = ...,
) -> AiofilesContextManager[None, None, AsyncFileIO]: ...

# Buffered binary reading/updating: AsyncBufferedReader
@overload
def open(
    file: _OpenFile,
    mode: OpenBinaryModeReading | OpenBinaryModeUpdating,
    buffering: Literal[-1, 1] = ...,
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
    closefd: bool = ...,
    opener: _Opener | None = ...,
    *,
    loop: AbstractEventLoop | None = ...,
    executor: Any | None = ...,
) -> AiofilesContextManager[None, None, AsyncBufferedReader]: ...

# Buffered binary writing: AsyncBufferedIOBase
@overload
def open(
    file: _OpenFile,
    mode: OpenBinaryModeWriting,
    buffering: Literal[-1, 1] = ...,
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
    closefd: bool = ...,
    opener: _Opener | None = ...,
    *,
    loop: AbstractEventLoop | None = ...,
    executor: Any | None = ...,
) -> AiofilesContextManager[None, None, AsyncBufferedIOBase]: ...

# Buffering cannot be determined: fall back to _UnknownAsyncBinaryIO
@overload
def open(
    file: _OpenFile,
    mode: OpenBinaryMode,
    buffering: int = ...,
    encoding: None = ...,
    errors: None = ...,
    newline: None = ...,
    closefd: bool = ...,
    opener: _Opener | None = ...,
    *,
    loop: AbstractEventLoop | None = ...,
    executor: Any | None = ...,
) -> AiofilesContextManager[None, None, _UnknownAsyncBinaryIO]: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable
from openpyxl.worksheet.protection import _Protected

class ChartsheetProtection(Serialisable, _Protected):
    tagname: str
    algorithmName: Any
    hashValue: Any
    saltValue: Any
    spinCount: Any
    content: Any
    objects: Any
    __attrs__: Any
    password: Any
    def __init__(
        self,
        content: Any | None = ...,
        objects: Any | None = ...,
        hashValue: Any | None = ...,
        spinCount: Any | None = ...,
        saltValue: Any | None = ...,
        algorithmName: Any | None = ...,
        password: Any | None = ...,
    ) -> None: ...



import re
from re import Match
from typing import Any, ClassVar
from xml.etree.ElementTree import Element

from markdown.core import Markdown

def build_inlinepatterns(md, **kwargs): ...

NOIMG: str
BACKTICK_RE: str
ESCAPE_RE: str
EMPHASIS_RE: str
STRONG_RE: str
SMART_STRONG_RE: str
SMART_EMPHASIS_RE: str
SMART_STRONG_EM_RE: str
EM_STRONG_RE: str
EM_STRONG2_RE: str
STRONG_EM_RE: str
STRONG_EM2_RE: str
STRONG_EM3_RE: str
LINK_RE: str
IMAGE_LINK_RE: str
REFERENCE_RE: str
IMAGE_REFERENCE_RE: str
NOT_STRONG_RE: str
AUTOLINK_RE: str
AUTOMAIL_RE: str
HTML_RE: str
ENTITY_RE: str
LINE_BREAK_RE: str

def dequote(string): ...

class EmStrongItem: ...

class Pattern:
    ANCESTOR_EXCLUDES: Any
    pattern: Any
    compiled_re: Any
    md: Markdown
    def __init__(self, pattern, md: Markdown | None = ...) -> None: ...
    def getCompiledRegExp(self): ...
    def handleMatch(self, m: Match[str]) -> str | Element | None: ...
    def type(self): ...
    def unescape(self, text): ...

class InlineProcessor(Pattern):
    safe_mode: bool = ...
    def __init__(self, pattern, md: Markdown | None = ...) -> None: ...
    def handleMatch(self, m: Match[str], data) -> tuple[Element, int, int] | tuple[None, None, None]: ...  # type: ignore[override]

class SimpleTextPattern(Pattern): ...
class SimpleTextInlineProcessor(InlineProcessor): ...
class EscapeInlineProcessor(InlineProcessor): ...

class SimpleTagPattern(Pattern):
    tag: Any
    def __init__(self, pattern, tag) -> None: ...

class SimpleTagInlineProcessor(InlineProcessor):
    tag: Any
    def __init__(self, pattern, tag) -> None: ...

class SubstituteTagPattern(SimpleTagPattern): ...
class SubstituteTagInlineProcessor(SimpleTagInlineProcessor): ...

class BacktickInlineProcessor(InlineProcessor):
    ESCAPED_BSLASH: Any
    tag: str = ...
    def __init__(self, pattern) -> None: ...

class DoubleTagPattern(SimpleTagPattern): ...
class DoubleTagInlineProcessor(SimpleTagInlineProcessor): ...
class HtmlInlineProcessor(InlineProcessor): ...

class AsteriskProcessor(InlineProcessor):
    PATTERNS: Any
    def build_single(self, m, tag, idx): ...
    def build_double(self, m, tags, idx): ...
    def build_double2(self, m, tags, idx): ...
    def parse_sub_patterns(self, data, parent, last, idx) -> None: ...
    def build_element(self, m, builder, tags, index): ...

class UnderscoreProcessor(AsteriskProcessor):
    PATTERNS: Any

class LinkInlineProcessor(InlineProcessor):
    RE_LINK: Any
    RE_TITLE_CLEAN: Any
    def getLink(self, data, index): ...
    def getText(self, data, index): ...

class ImageInlineProcessor(LinkInlineProcessor): ...

class ReferenceInlineProcessor(LinkInlineProcessor):
    NEWLINE_CLEANUP_RE: ClassVar[re.Pattern[str]]
    def evalId(self, data, index, text): ...
    def makeTag(self, href, title, text): ...

class ShortReferenceInlineProcessor(ReferenceInlineProcessor): ...
class ImageReferenceInlineProcessor(ReferenceInlineProcessor): ...
class AutolinkInlineProcessor(InlineProcessor): ...
class AutomailInlineProcessor(InlineProcessor): ...



from _typeshed import Incomplete

class BaseQuery:
    counters: Incomplete
    paths: Incomplete
    active: int
    curpaths: Incomplete
    def __init__(self, paths: Incomplete | None = ...) -> None: ...
    def addcounterbybrowsing(self, flags=..., windowtitle: str = ...) -> None: ...
    def rawaddcounter(
        self, object, counter, instance: Incomplete | None = ..., inum: int = ..., machine: Incomplete | None = ...
    ) -> None: ...
    def addcounter(
        self, object, counter, instance: Incomplete | None = ..., inum: int = ..., machine: Incomplete | None = ...
    ): ...
    def open(self): ...
    def killbase(self, base: Incomplete | None = ...) -> None: ...
    def close(self) -> None: ...
    __del__: Incomplete
    def collectdata(self, format=...): ...
    def collectdataslave(self, format=...): ...
    def __getinitargs__(self): ...

class Query(BaseQuery):
    volatilecounters: Incomplete
    def __init__(self, *args, **namedargs) -> None: ...
    def addinstcounter(
        self, object, counter, machine: Incomplete | None = ..., objtype: str = ..., volatile: int = ..., format=...
    ) -> None: ...
    def getinstpaths(self, object, counter, machine: Incomplete | None = ..., objtype: str = ..., format=...): ...
    def open(self, *args, **namedargs) -> None: ...
    curresults: Incomplete
    def collectdatafor(self, totalperiod, period: int = ...) -> None: ...
    collectdatawhile_active: int
    def collectdatawhile(self, period: int = ...) -> None: ...
    def collectdatawhile_stop(self) -> None: ...
    def collectdatawhile_slave(self, period) -> None: ...
    def __getinitargs__(self): ...

class QueryError:
    query: Incomplete
    def __init__(self, query) -> None: ...



from typing import Any

from markdown.core import Markdown
from markdown.extensions import Extension
from markdown.inlinepatterns import InlineProcessor

def build_url(label, base, end): ...

class WikiLinkExtension(Extension):
    def __init__(self, **kwargs) -> None: ...
    md: Markdown

class WikiLinksInlineProcessor(InlineProcessor):
    config: Any
    def __init__(self, pattern, config) -> None: ...

def makeExtension(**kwargs): ...



from typing import Any
from typing_extensions import TypeAlias

from . import util

_SemanticAnalyzerPluginInterface: TypeAlias = Any  # mypy.plugin.SemanticAnalyzerPluginInterface

def scan_declarative_assignments_and_apply_types(
    cls, api: _SemanticAnalyzerPluginInterface, is_mixin_scan: bool = ...
) -> list[util.SQLAlchemyAttribute] | None: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

def get_version(): ...

class DigSigBlob(Serialisable):  # type: ignore[misc]
    __elements__: Any
    __attrs__: Any

class VectorLpstr(Serialisable):  # type: ignore[misc]
    __elements__: Any
    __attrs__: Any

class VectorVariant(Serialisable):  # type: ignore[misc]
    __elements__: Any
    __attrs__: Any

class ExtendedProperties(Serialisable):
    tagname: str
    Template: Any
    Manager: Any
    Company: Any
    Pages: Any
    Words: Any
    Characters: Any
    PresentationFormat: Any
    Lines: Any
    Paragraphs: Any
    Slides: Any
    Notes: Any
    TotalTime: Any
    HiddenSlides: Any
    MMClips: Any
    ScaleCrop: Any
    HeadingPairs: Any
    TitlesOfParts: Any
    LinksUpToDate: Any
    CharactersWithSpaces: Any
    SharedDoc: Any
    HyperlinkBase: Any
    HLinks: Any
    HyperlinksChanged: Any
    DigSig: Any
    Application: Any
    AppVersion: Any
    DocSecurity: Any
    __elements__: Any
    def __init__(
        self,
        Template: Any | None = ...,
        Manager: Any | None = ...,
        Company: Any | None = ...,
        Pages: Any | None = ...,
        Words: Any | None = ...,
        Characters: Any | None = ...,
        PresentationFormat: Any | None = ...,
        Lines: Any | None = ...,
        Paragraphs: Any | None = ...,
        Slides: Any | None = ...,
        Notes: Any | None = ...,
        TotalTime: Any | None = ...,
        HiddenSlides: Any | None = ...,
        MMClips: Any | None = ...,
        ScaleCrop: Any | None = ...,
        HeadingPairs: Any | None = ...,
        TitlesOfParts: Any | None = ...,
        LinksUpToDate: Any | None = ...,
        CharactersWithSpaces: Any | None = ...,
        SharedDoc: Any | None = ...,
        HyperlinkBase: Any | None = ...,
        HLinks: Any | None = ...,
        HyperlinksChanged: Any | None = ...,
        DigSig: Any | None = ...,
        Application: str = ...,
        AppVersion: Any | None = ...,
        DocSecurity: Any | None = ...,
    ) -> None: ...
    def to_tree(self): ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import ClassVar

from .prettytable import PrettyTable

RESET_CODE: str

init: Callable[[], object]

class Theme:
    default_color: str
    vertical_char: str
    vertical_color: str
    horizontal_char: str
    horizontal_color: str
    junction_char: str
    junction_color: str
    def __init__(
        self,
        default_color: str = ...,
        vertical_char: str = ...,
        vertical_color: str = ...,
        horizontal_char: str = ...,
        horizontal_color: str = ...,
        junction_char: str = ...,
        junction_color: str = ...,
    ) -> None: ...
    # The following method is broken in upstream code.
    def format_code(s: str) -> str: ...  # type: ignore[misc]

class Themes:
    DEFAULT: ClassVar[Theme]
    OCEAN: ClassVar[Theme]

class ColorTable(PrettyTable):
    def __init__(self, field_names: Incomplete | None = ..., **kwargs) -> None: ...
    @property
    def theme(self) -> Theme: ...
    @theme.setter
    def theme(self, value: Theme): ...
    def update_theme(self) -> None: ...
    def get_string(self, **kwargs) -> str: ...



import ctypes
from ctypes import wintypes

class DISPLAY_DEVICE(ctypes.Structure):
    cb: wintypes.DWORD
    DeviceName: wintypes.WCHAR
    DeviceString: wintypes.WCHAR
    StateFlags: wintypes.DWORD
    DeviceID: wintypes.WCHAR
    DeviceKey: wintypes.WCHAR

def get_display_device_name_mapping() -> dict[str, tuple[str, bool]]: ...
def get_hmonitor_by_point(x: wintypes.LONG, y: wintypes.LONG) -> wintypes.HMONITOR: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class EmbeddedWAVAudioFile(Serialisable):  # type: ignore[misc]
    name: Any
    def __init__(self, name: Any | None = ...) -> None: ...

class Hyperlink(Serialisable):
    tagname: str
    namespace: Any
    invalidUrl: Any
    action: Any
    tgtFrame: Any
    tooltip: Any
    history: Any
    highlightClick: Any
    endSnd: Any
    snd: Any
    extLst: Any
    id: Any
    __elements__: Any
    def __init__(
        self,
        invalidUrl: Any | None = ...,
        action: Any | None = ...,
        tgtFrame: Any | None = ...,
        tooltip: Any | None = ...,
        history: Any | None = ...,
        highlightClick: Any | None = ...,
        endSnd: Any | None = ...,
        snd: Any | None = ...,
        extLst: Any | None = ...,
        id: Any | None = ...,
    ) -> None: ...

class Font(Serialisable):
    tagname: str
    namespace: Any
    typeface: Any
    panose: Any
    pitchFamily: Any
    charset: Any
    def __init__(
        self, typeface: Any | None = ..., panose: Any | None = ..., pitchFamily: Any | None = ..., charset: Any | None = ...
    ) -> None: ...

class CharacterProperties(Serialisable):
    tagname: str
    namespace: Any
    kumimoji: Any
    lang: Any
    altLang: Any
    sz: Any
    b: Any
    i: Any
    u: Any
    strike: Any
    kern: Any
    cap: Any
    spc: Any
    normalizeH: Any
    baseline: Any
    noProof: Any
    dirty: Any
    err: Any
    smtClean: Any
    smtId: Any
    bmk: Any
    ln: Any
    highlight: Any
    latin: Any
    ea: Any
    cs: Any
    sym: Any
    hlinkClick: Any
    hlinkMouseOver: Any
    rtl: Any
    extLst: Any
    noFill: Any
    solidFill: Any
    gradFill: Any
    blipFill: Any
    pattFill: Any
    grpFill: Any
    effectLst: Any
    effectDag: Any
    uLnTx: Any
    uLn: Any
    uFillTx: Any
    uFill: Any
    __elements__: Any
    def __init__(
        self,
        kumimoji: Any | None = ...,
        lang: Any | None = ...,
        altLang: Any | None = ...,
        sz: Any | None = ...,
        b: Any | None = ...,
        i: Any | None = ...,
        u: Any | None = ...,
        strike: Any | None = ...,
        kern: Any | None = ...,
        cap: Any | None = ...,
        spc: Any | None = ...,
        normalizeH: Any | None = ...,
        baseline: Any | None = ...,
        noProof: Any | None = ...,
        dirty: Any | None = ...,
        err: Any | None = ...,
        smtClean: Any | None = ...,
        smtId: Any | None = ...,
        bmk: Any | None = ...,
        ln: Any | None = ...,
        highlight: Any | None = ...,
        latin: Any | None = ...,
        ea: Any | None = ...,
        cs: Any | None = ...,
        sym: Any | None = ...,
        hlinkClick: Any | None = ...,
        hlinkMouseOver: Any | None = ...,
        rtl: Any | None = ...,
        extLst: Any | None = ...,
        noFill: Any | None = ...,
        solidFill: Any | None = ...,
        gradFill: Any | None = ...,
        blipFill: Any | None = ...,
        pattFill: Any | None = ...,
        grpFill: Any | None = ...,
        effectLst: Any | None = ...,
        effectDag: Any | None = ...,
        uLnTx: Any | None = ...,
        uLn: Any | None = ...,
        uFillTx: Any | None = ...,
        uFill: Any | None = ...,
    ) -> None: ...

class TabStop(Serialisable):  # type: ignore[misc]
    pos: Any
    algn: Any
    def __init__(self, pos: Any | None = ..., algn: Any | None = ...) -> None: ...

class TabStopList(Serialisable):  # type: ignore[misc]
    tab: Any
    def __init__(self, tab: Any | None = ...) -> None: ...

class Spacing(Serialisable):
    spcPct: Any
    spcPts: Any
    __elements__: Any
    def __init__(self, spcPct: Any | None = ..., spcPts: Any | None = ...) -> None: ...

class AutonumberBullet(Serialisable):
    type: Any
    startAt: Any
    def __init__(self, type: Any | None = ..., startAt: Any | None = ...) -> None: ...

class ParagraphProperties(Serialisable):
    tagname: str
    namespace: Any
    marL: Any
    marR: Any
    lvl: Any
    indent: Any
    algn: Any
    defTabSz: Any
    rtl: Any
    eaLnBrk: Any
    fontAlgn: Any
    latinLnBrk: Any
    hangingPunct: Any
    lnSpc: Any
    spcBef: Any
    spcAft: Any
    tabLst: Any
    defRPr: Any
    extLst: Any
    buClrTx: Any
    buClr: Any
    buSzTx: Any
    buSzPct: Any
    buSzPts: Any
    buFontTx: Any
    buFont: Any
    buNone: Any
    buAutoNum: Any
    buChar: Any
    buBlip: Any
    __elements__: Any
    def __init__(
        self,
        marL: Any | None = ...,
        marR: Any | None = ...,
        lvl: Any | None = ...,
        indent: Any | None = ...,
        algn: Any | None = ...,
        defTabSz: Any | None = ...,
        rtl: Any | None = ...,
        eaLnBrk: Any | None = ...,
        fontAlgn: Any | None = ...,
        latinLnBrk: Any | None = ...,
        hangingPunct: Any | None = ...,
        lnSpc: Any | None = ...,
        spcBef: Any | None = ...,
        spcAft: Any | None = ...,
        tabLst: Any | None = ...,
        defRPr: Any | None = ...,
        extLst: Any | None = ...,
        buClrTx: Any | None = ...,
        buClr: Any | None = ...,
        buSzTx: Any | None = ...,
        buSzPct: Any | None = ...,
        buSzPts: Any | None = ...,
        buFontTx: Any | None = ...,
        buFont: Any | None = ...,
        buNone: Any | None = ...,
        buAutoNum: Any | None = ...,
        buChar: Any | None = ...,
        buBlip: Any | None = ...,
    ) -> None: ...

class ListStyle(Serialisable):
    tagname: str
    namespace: Any
    defPPr: Any
    lvl1pPr: Any
    lvl2pPr: Any
    lvl3pPr: Any
    lvl4pPr: Any
    lvl5pPr: Any
    lvl6pPr: Any
    lvl7pPr: Any
    lvl8pPr: Any
    lvl9pPr: Any
    extLst: Any
    __elements__: Any
    def __init__(
        self,
        defPPr: Any | None = ...,
        lvl1pPr: Any | None = ...,
        lvl2pPr: Any | None = ...,
        lvl3pPr: Any | None = ...,
        lvl4pPr: Any | None = ...,
        lvl5pPr: Any | None = ...,
        lvl6pPr: Any | None = ...,
        lvl7pPr: Any | None = ...,
        lvl8pPr: Any | None = ...,
        lvl9pPr: Any | None = ...,
        extLst: Any | None = ...,
    ) -> None: ...

class RegularTextRun(Serialisable):
    tagname: str
    namespace: Any
    rPr: Any
    properties: Any
    t: Any
    value: Any
    __elements__: Any
    def __init__(self, rPr: Any | None = ..., t: str = ...) -> None: ...

class LineBreak(Serialisable):
    tagname: str
    namespace: Any
    rPr: Any
    __elements__: Any
    def __init__(self, rPr: Any | None = ...) -> None: ...

class TextField(Serialisable):
    id: Any
    type: Any
    rPr: Any
    pPr: Any
    t: Any
    __elements__: Any
    def __init__(
        self, id: Any | None = ..., type: Any | None = ..., rPr: Any | None = ..., pPr: Any | None = ..., t: Any | None = ...
    ) -> None: ...

class Paragraph(Serialisable):
    tagname: str
    namespace: Any
    pPr: Any
    properties: Any
    endParaRPr: Any
    r: Any
    text: Any
    br: Any
    fld: Any
    __elements__: Any
    def __init__(
        self,
        pPr: Any | None = ...,
        endParaRPr: Any | None = ...,
        r: Any | None = ...,
        br: Any | None = ...,
        fld: Any | None = ...,
    ) -> None: ...

class GeomGuide(Serialisable):
    name: Any
    fmla: Any
    def __init__(self, name: Any | None = ..., fmla: Any | None = ...) -> None: ...

class GeomGuideList(Serialisable):
    gd: Any
    def __init__(self, gd: Any | None = ...) -> None: ...

class PresetTextShape(Serialisable):
    prst: Any
    avLst: Any
    def __init__(self, prst: Any | None = ..., avLst: Any | None = ...) -> None: ...

class TextNormalAutofit(Serialisable):
    fontScale: Any
    lnSpcReduction: Any
    def __init__(self, fontScale: Any | None = ..., lnSpcReduction: Any | None = ...) -> None: ...

class RichTextProperties(Serialisable):
    tagname: str
    namespace: Any
    rot: Any
    spcFirstLastPara: Any
    vertOverflow: Any
    horzOverflow: Any
    vert: Any
    wrap: Any
    lIns: Any
    tIns: Any
    rIns: Any
    bIns: Any
    numCol: Any
    spcCol: Any
    rtlCol: Any
    fromWordArt: Any
    anchor: Any
    anchorCtr: Any
    forceAA: Any
    upright: Any
    compatLnSpc: Any
    prstTxWarp: Any
    scene3d: Any
    extLst: Any
    noAutofit: Any
    normAutofit: Any
    spAutoFit: Any
    flatTx: Any
    __elements__: Any
    def __init__(
        self,
        rot: Any | None = ...,
        spcFirstLastPara: Any | None = ...,
        vertOverflow: Any | None = ...,
        horzOverflow: Any | None = ...,
        vert: Any | None = ...,
        wrap: Any | None = ...,
        lIns: Any | None = ...,
        tIns: Any | None = ...,
        rIns: Any | None = ...,
        bIns: Any | None = ...,
        numCol: Any | None = ...,
        spcCol: Any | None = ...,
        rtlCol: Any | None = ...,
        fromWordArt: Any | None = ...,
        anchor: Any | None = ...,
        anchorCtr: Any | None = ...,
        forceAA: Any | None = ...,
        upright: Any | None = ...,
        compatLnSpc: Any | None = ...,
        prstTxWarp: Any | None = ...,
        scene3d: Any | None = ...,
        extLst: Any | None = ...,
        noAutofit: Any | None = ...,
        normAutofit: Any | None = ...,
        spAutoFit: Any | None = ...,
        flatTx: Any | None = ...,
    ) -> None: ...



import _win32typing

CATID_ActiveScript: _win32typing.PyIID
CATID_ActiveScriptParse: _win32typing.PyIID
IID_IActiveScript: _win32typing.PyIID
IID_IActiveScriptError: _win32typing.PyIID
IID_IActiveScriptParse: _win32typing.PyIID
IID_IActiveScriptParseProcedure: _win32typing.PyIID
IID_IActiveScriptSite: _win32typing.PyIID
IID_IObjectSafety: _win32typing.PyIID
IID_IProvideMultipleClassInfo: _win32typing.PyIID
INTERFACESAFE_FOR_UNTRUSTED_CALLER: int
INTERFACESAFE_FOR_UNTRUSTED_DATA: int
INTERFACE_USES_DISPEX: int
INTERFACE_USES_SECURITY_MANAGER: int
MULTICLASSINFO_GETIIDPRIMARY: int
MULTICLASSINFO_GETIIDSOURCE: int
MULTICLASSINFO_GETNUMRESERVEDDISPIDS: int
MULTICLASSINFO_GETTYPEINFO: int
SCRIPTINFO_ALL_FLAGS: int
SCRIPTINFO_ITYPEINFO: int
SCRIPTINFO_IUNKNOWN: int
SCRIPTINTERRUPT_ALL_FLAGS: int
SCRIPTINTERRUPT_DEBUG: int
SCRIPTINTERRUPT_RAISEEXCEPTION: int
SCRIPTITEM_ALL_FLAGS: int
SCRIPTITEM_CODEONLY: int
SCRIPTITEM_GLOBALMEMBERS: int
SCRIPTITEM_ISPERSISTENT: int
SCRIPTITEM_ISSOURCE: int
SCRIPTITEM_ISVISIBLE: int
SCRIPTITEM_NOCODE: int
SCRIPTPROC_ALL_FLAGS: int
SCRIPTPROC_HOSTMANAGESSOURCE: int
SCRIPTPROC_IMPLICIT_PARENTS: int
SCRIPTPROC_IMPLICIT_THIS: int
SCRIPTSTATE_CLOSED: int
SCRIPTSTATE_CONNECTED: int
SCRIPTSTATE_DISCONNECTED: int
SCRIPTSTATE_INITIALIZED: int
SCRIPTSTATE_STARTED: int
SCRIPTSTATE_UNINITIALIZED: int
SCRIPTTEXT_ALL_FLAGS: int
SCRIPTTEXT_ISEXPRESSION: int
SCRIPTTEXT_ISPERSISTENT: int
SCRIPTTEXT_ISVISIBLE: int
SCRIPTTHREADSTATE_NOTINSCRIPT: int
SCRIPTTHREADSTATE_RUNNING: int
SCRIPTTYPELIB_ISCONTROL: int
SCRIPTTYPELIB_ISPERSISTENT: int
SCRIPT_E_REPORTED: int
TIFLAGS_EXTENDDISPATCHONLY: int



from typing import Any

from .backends.base import Key

def encrypt(
    plaintext: str | bytes,
    # Internally it's passed down to jwk.construct(), which explicitly checks for
    # key as dict instance, instead of a Mapping
    key: str | bytes | dict[str, Any] | Key,
    encryption: str = ...,
    algorithm: str = ...,
    zip: str | None = ...,
    cty: str | None = ...,
    kid: str | None = ...,
) -> bytes: ...
def decrypt(
    jwe_str: str | bytes,
    # Internally it's passed down to jwk.construct(), which explicitly checks for
    # key as dict instance, instead of a Mapping
    key: str | bytes | dict[str, Any] | Key,
) -> bytes | None: ...
def get_unverified_header(jwe_str: str | bytes | None) -> dict[str, Any]: ...



from _typeshed import Incomplete

from win32com.server.exception import COMException as COMException
from win32comext.axdebug import gateways
from win32comext.axdebug.util import RaiseNotImpl as RaiseNotImpl, trace as trace

class EnumDebugStackFrames(gateways.EnumDebugStackFrames):
    def __init__(self, debugger) -> None: ...
    def Next(self, count): ...

class DebugStackFrame(gateways.DebugStackFrame):
    frame: Incomplete
    lineno: Incomplete
    codeContainer: Incomplete
    expressionContext: Incomplete
    def __init__(self, frame, lineno, codeContainer) -> None: ...
    def GetThread(self) -> None: ...
    def GetCodeContext(self): ...
    def GetDescriptionString(self, fLong): ...
    def GetLanguageString(self, fLong): ...
    def GetDebugProperty(self): ...

class DebugStackFrameSniffer:
    debugger: Incomplete
    def __init__(self, debugger) -> None: ...
    def EnumStackFrames(self): ...

class StackFrameDebugProperty:
    frame: Incomplete
    def __init__(self, frame) -> None: ...
    def GetPropertyInfo(self, dwFieldSpec, nRadix) -> None: ...
    def GetExtendedInfo(self) -> None: ...
    def SetValueAsString(self, value, radix) -> None: ...
    def EnumMembers(self, dwFieldSpec, nRadix, iid): ...
    def GetParent(self) -> None: ...



from _typeshed import Self
from typing import Any
from typing_extensions import Literal

from stripe.api_resources.abstract import CreateableAPIResource, DeletableAPIResource, ListableAPIResource

class TestClock(CreateableAPIResource, DeletableAPIResource, ListableAPIResource):
    OBJECT_NAME: Literal["test_helpers.test_clock"]

    @classmethod
    def advance(cls: type[Self], idempotency_key: str | None = ..., **params: Any) -> Self: ...



from collections.abc import Iterable, Iterator
from typing import Any

from pygments.lexer import Lexer
from pygments.token import _TokenType

def apply_filters(stream, filters, lexer: Any | None = ...): ...
def simplefilter(f): ...

class Filter:
    options: Any
    def __init__(self, **options) -> None: ...
    def filter(self, lexer: Lexer, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...

class FunctionFilter(Filter):
    function: Any
    def __init__(self, **options) -> None: ...
    def filter(self, lexer: Lexer, stream: Iterable[tuple[_TokenType, str]]) -> Iterator[tuple[_TokenType, str]]: ...



from typing import Any

from openpyxl.descriptors.serialisable import Serialisable

class Record(Serialisable):
    tagname: str
    m: Any
    n: Any
    b: Any
    e: Any
    s: Any
    d: Any
    x: Any
    def __init__(
        self,
        _fields=...,
        m: Any | None = ...,
        n: Any | None = ...,
        b: Any | None = ...,
        e: Any | None = ...,
        s: Any | None = ...,
        d: Any | None = ...,
        x: Any | None = ...,
    ) -> None: ...

class RecordList(Serialisable):
    mime_type: str
    rel_type: str
    tagname: str
    r: Any
    extLst: Any
    __elements__: Any
    __attrs__: Any
    def __init__(self, count: Any | None = ..., r=..., extLst: Any | None = ...) -> None: ...
    @property
    def count(self): ...
    def to_tree(self): ...
    @property
    def path(self): ...



from typing import Any

class IdentityMap:
    def keys(self): ...
    def replace(self, state) -> None: ...
    def add(self, state) -> None: ...
    def update(self, dict_) -> None: ...
    def clear(self) -> None: ...
    def check_modified(self): ...
    def has_key(self, key): ...
    def popitem(self) -> None: ...
    def pop(self, key, *args) -> None: ...
    def setdefault(self, key, default: Any | None = ...) -> None: ...
    def __len__(self): ...
    def copy(self) -> None: ...
    def __setitem__(self, key, value) -> None: ...
    def __delitem__(self, key) -> None: ...

class WeakInstanceDict(IdentityMap):
    def __getitem__(self, key): ...
    def __contains__(self, key): ...
    def contains_state(self, state): ...
    def replace(self, state): ...
    def add(self, state): ...
    def get(self, key, default: Any | None = ...): ...
    def items(self): ...
    def values(self): ...
    def __iter__(self): ...
    def all_states(self): ...
    def discard(self, state) -> None: ...
    def safe_discard(self, state) -> None: ...



from typing import Any

class ExtendedOperationContainer:
    def __init__(self, connection) -> None: ...

class StandardExtendedOperations(ExtendedOperationContainer):
    def who_am_i(self, controls: Any | None = ...): ...
    def modify_password(
        self,
        user: Any | None = ...,
        old_password: Any | None = ...,
        new_password: Any | None = ...,
        hash_algorithm: Any | None = ...,
        salt: Any | None = ...,
        controls: Any | None = ...,
    ): ...
    def paged_search(
        self,
        search_base,
        search_filter,
        search_scope=...,
        dereference_aliases=...,
        attributes: Any | None = ...,
        size_limit: int = ...,
        time_limit: int = ...,
        types_only: bool = ...,
        get_operational_attributes: bool = ...,
        controls: Any | None = ...,
        paged_size: int = ...,
        paged_criticality: bool = ...,
        generator: bool = ...,
    ): ...
    def persistent_search(
        self,
        search_base: str = ...,
        search_filter: str = ...,
        search_scope=...,
        dereference_aliases=...,
        attributes=...,
        size_limit: int = ...,
        time_limit: int = ...,
        controls: Any | None = ...,
        changes_only: bool = ...,
        show_additions: bool = ...,
        show_deletions: bool = ...,
        show_modifications: bool = ...,
        show_dn_modifications: bool = ...,
        notifications: bool = ...,
        streaming: bool = ...,
        callback: Any | None = ...,
    ): ...
    def funnel_search(
        self,
        search_base: str = ...,
        search_filter: str = ...,
        search_scope=...,
        dereference_aliases=...,
        attributes=...,
        size_limit: int = ...,
        time_limit: int = ...,
        controls: Any | None = ...,
        streaming: bool = ...,
        callback: Any | None = ...,
    ): ...

class NovellExtendedOperations(ExtendedOperationContainer):
    def get_bind_dn(self, controls: Any | None = ...): ...
    def get_universal_password(self, user, controls: Any | None = ...): ...
    def set_universal_password(self, user, new_password: Any | None = ..., controls: Any | None = ...): ...
    def list_replicas(self, server_dn, controls: Any | None = ...): ...
    def partition_entry_count(self, partition_dn, controls: Any | None = ...): ...
    def replica_info(self, server_dn, partition_dn, controls: Any | None = ...): ...
    def start_transaction(self, controls: Any | None = ...): ...
    def end_transaction(self, commit: bool = ..., controls: Any | None = ...): ...
    def add_members_to_groups(self, members, groups, fix: bool = ..., transaction: bool = ...): ...
    def remove_members_from_groups(self, members, groups, fix: bool = ..., transaction: bool = ...): ...
    def check_groups_memberships(self, members, groups, fix: bool = ..., transaction: bool = ...): ...

class MicrosoftExtendedOperations(ExtendedOperationContainer):
    def dir_sync(
        self,
        sync_base,
        sync_filter: str = ...,
        attributes=...,
        cookie: Any | None = ...,
        object_security: bool = ...,
        ancestors_first: bool = ...,
        public_data_only: bool = ...,
        incremental_values: bool = ...,
        max_length: int = ...,
        hex_guid: bool = ...,
    ): ...
    def modify_password(self, user, new_password, old_password: Any | None = ..., controls: Any | None = ...): ...
    def unlock_account(self, user): ...
    def add_members_to_groups(self, members, groups, fix: bool = ...): ...
    def remove_members_from_groups(self, members, groups, fix: bool = ...): ...
    def persistent_search(
        self, search_base: str = ..., search_scope=..., attributes=..., streaming: bool = ..., callback: Any | None = ...
    ): ...

class ExtendedOperationsRoot(ExtendedOperationContainer):
    standard: Any
    novell: Any
    microsoft: Any
    def __init__(self, connection) -> None: ...



import argparse
from typing import Any

def main_inner(parser, argns): ...

class HelpFormatter(argparse.HelpFormatter):
    def __init__(self, prog, indent_increment: int = ..., max_help_position: int = ..., width: Any | None = ...) -> None: ...

def main(args=...): ...



def patternProperties(validator, patternProperties, instance, schema) -> None: ...
def propertyNames(validator, propertyNames, instance, schema) -> None: ...
def additionalProperties(validator, aP, instance, schema) -> None: ...
def items(validator, items, instance, schema) -> None: ...
def additionalItems(validator, aI, instance, schema) -> None: ...
def const(validator, const, instance, schema) -> None: ...
def contains(validator, contains, instance, schema) -> None: ...
def exclusiveMinimum(validator, minimum, instance, schema) -> None: ...
def exclusiveMaximum(validator, maximum, instance, schema) -> None: ...
def minimum(validator, minimum, instance, schema) -> None: ...
def maximum(validator, maximum, instance, schema) -> None: ...
def multipleOf(validator, dB, instance, schema) -> None: ...
def minItems(validator, mI, instance, schema) -> None: ...
def maxItems(validator, mI, instance, schema) -> None: ...
def uniqueItems(validator, uI, instance, schema) -> None: ...
def pattern(validator, patrn, instance, schema) -> None: ...
def format(validator, format, instance, schema) -> None: ...
def minLength(validator, mL, instance, schema) -> None: ...
def maxLength(validator, mL, instance, schema) -> None: ...
def dependentRequired(validator, dependentRequired, instance, schema) -> None: ...
def dependentSchemas(validator, dependentSchemas, instance, schema) -> None: ...
def enum(validator, enums, instance, schema) -> None: ...
def ref(validator, ref, instance, schema) -> None: ...
def dynamicRef(validator, dynamicRef, instance, schema) -> None: ...
def type(validator, types, instance, schema) -> None: ...
def properties(validator, properties, instance, schema) -> None: ...
def required(validator, required, instance, schema) -> None: ...
def minProperties(validator, mP, instance, schema) -> None: ...
def maxProperties(validator, mP, instance, schema) -> None: ...
def allOf(validator, allOf, instance, schema) -> None: ...
def anyOf(validator, anyOf, instance, schema) -> None: ...
def oneOf(validator, oneOf, instance, schema) -> None: ...
def not_(validator, not_schema, instance, schema) -> None: ...
def if_(validator, if_schema, instance, schema) -> None: ...
def unevaluatedItems(validator, unevaluatedItems, instance, schema) -> None: ...
def unevaluatedProperties(validator, unevaluatedProperties, instance, schema) -> None: ...
def prefixItems(validator, prefixItems, instance, schema) -> None: ...



from collections.abc import Iterable, Mapping
from typing import Any, TextIO, overload
from typing_extensions import Literal, TypeAlias

from .watchers import StreamWatcher

_Hide: TypeAlias = Literal[None, True, False, "out", "stdout", "err", "stderr", "both"]

class Runner:
    read_chunk_size: int
    input_sleep: float
    context: Any
    program_finished: Any
    warned_about_pty_fallback: bool
    watchers: Any
    def __init__(self, context) -> None: ...
    # If disown is True (default=False), returns None
    @overload
    def run(
        self,
        command: str,
        *,
        asynchronous: bool = ...,
        disown: Literal[True],
        dry: bool = ...,
        echo: bool = ...,
        echo_format: str = ...,
        echo_stdin: bool | None = ...,
        encoding: str = ...,
        err_stream: TextIO | None = ...,
        env: Mapping[str, str] = ...,
        fallback: bool = ...,
        hide: _Hide = ...,
        in_stream: TextIO | None | bool = ...,
        out_stream: TextIO | None = ...,
        pty: bool = ...,
        replace_env: bool = ...,
        shell: str = ...,
        timeout: float | None = ...,
        warn: bool = ...,
        watchers: Iterable[StreamWatcher] = ...,
    ) -> None: ...
    # If disown is False (the default), and asynchronous is True (default=False) returns Promise
    @overload
    def run(
        self,
        command: str,
        *,
        asynchronous: Literal[True],
        disown: Literal[False] = ...,
        dry: bool = ...,
        echo: bool = ...,
        echo_format: str = ...,
        echo_stdin: bool | None = ...,
        encoding: str = ...,
        err_stream: TextIO | None = ...,
        env: Mapping[str, str] = ...,
        fallback: bool = ...,
        hide: _Hide = ...,
        in_stream: TextIO | None | bool = ...,
        out_stream: TextIO | None = ...,
        pty: bool = ...,
        replace_env: bool = ...,
        shell: str = ...,
        timeout: float | None = ...,
        warn: bool = ...,
        watchers: Iterable[StreamWatcher] = ...,
    ) -> Promise: ...
    # If disown and asynchronous are both False (the defaults), returns Result
    @overload
    def run(
        self,
        command: str,
        *,
        asynchronous: Literal[False] = ...,
        disown: Literal[False] = ...,
        dry: bool = ...,
        echo: bool = ...,
        echo_format: str = ...,
        echo_stdin: bool | None = ...,
        encoding: str = ...,
        err_stream: TextIO | None = ...,
        env: Mapping[str, str] = ...,
        fallback: bool = ...,
        hide: _Hide = ...,
        in_stream: TextIO | None | bool = ...,
        out_stream: TextIO | None = ...,
        pty: bool = ...,
        replace_env: bool = ...,
        shell: str = ...,
        timeout: float | None = ...,
        warn: bool = ...,
        watchers: Iterable[StreamWatcher] = ...,
    ) -> Result: ...
    # Fallback overload: return Any
    @overload
    def run(
        self,
        command: str,
        *,
        asynchronous: bool,
        disown: bool,
        dry: bool = ...,
        echo: bool = ...,
        echo_format: str = ...,
        echo_stdin: bool | None = ...,
        encoding: str = ...,
        err_stream: TextIO | None = ...,
        env: Mapping[str, str] = ...,
        fallback: bool = ...,
        hide: _Hide = ...,
        in_stream: TextIO | None | bool = ...,
        out_stream: TextIO | None = ...,
        pty: bool = ...,
        replace_env: bool = ...,
        shell: str = ...,
        timeout: float | None = ...,
        warn: bool = ...,
        watchers: Iterable[StreamWatcher] = ...,
    ) -> Any: ...
    def echo(self, command) -> None: ...
    def make_promise(self): ...
    def create_io_threads(self): ...
    def generate_result(self, **kwargs): ...
    def read_proc_output(self, reader) -> None: ...
    def write_our_output(self, stream, string) -> None: ...
    def handle_stdout(self, buffer_, hide, output) -> None: ...
    def handle_stderr(self, buffer_, hide, output) -> None: ...
    def read_our_stdin(self, input_): ...
    def handle_stdin(self, input_, output, echo) -> None: ...
    def should_echo_stdin(self, input_, output): ...
    def respond(self, buffer_) -> None: ...
    def generate_env(self, env, replace_env): ...
    def should_use_pty(self, pty, fallback): ...
    @property
    def has_dead_threads(self): ...
    def wait(self) -> None: ...
    def write_proc_stdin(self, data) -> None: ...
    def decode(self, data): ...
    @property
    def process_is_finished(self) -> None: ...
    def start(self, command, shell, env) -> None: ...
    def start_timer(self, timeout) -> None: ...
    def read_proc_stdout(self, num_bytes) -> None: ...
    def read_proc_stderr(self, num_bytes) -> None: ...
    def close_proc_stdin(self) -> None: ...
    def default_encoding(self): ...
    def send_interrupt(self, interrupt) -> None: ...
    def returncode(self) -> None: ...
    def stop(self) -> None: ...
    def stop_timer(self) -> None: ...
    def kill(self) -> None: ...
    @property
    def timed_out(self): ...

class Local(Runner):
    status: Any
    def __init__(self, context) -> None: ...
    def should_use_pty(self, pty: bool = ..., fallback: bool = ...): ...
    process: Any

class Result:
    stdout: str
    stderr: str
    encoding: str
    command: str
    shell: Any
    env: dict[str, Any]
    exited: int
    pty: bool
    hide: tuple[Literal["stdout", "stderr"], ...]
    def __init__(
        self,
        stdout: str = ...,
        stderr: str = ...,
        encoding: str | None = ...,
        command: str = ...,
        shell: str = ...,
        env=...,
        exited: int = ...,
        pty: bool = ...,
        hide: tuple[Literal["stdout", "stderr"], ...] = ...,
    ) -> None: ...
    @property
    def return_code(self) -> int: ...
    def __nonzero__(self) -> bool: ...
    def __bool__(self) -> bool: ...
    @property
    def ok(self) -> bool: ...
    @property
    def failed(self) -> bool: ...
    def tail(self, stream: Literal["stderr", "stdout"], count: int = ...) -> str: ...

class Promise(Result):
    runner: Any
    def __init__(self, runner) -> None: ...
    def join(self): ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback) -> None: ...

def normalize_hide(val, out_stream=..., err_stream=...): ...
def default_encoding() -> str: ...



from typing import Any

class FPDFRecorder:
    pdf: Any
    accept_page_break: bool
    def __init__(self, pdf, accept_page_break: bool = ...) -> None: ...
    def __getattr__(self, name): ...
    def rewind(self) -> None: ...
    def replay(self) -> None: ...

class CallRecorder:
    def __init__(self, func, calls) -> None: ...
    def __call__(self, *args, **kwargs): ...



import datetime
import http.client
import ssl
import sys
from _typeshed import ReadableBuffer, StrPath
from collections.abc import Callable
from logging import FileHandler, Handler, LogRecord
from queue import Queue, SimpleQueue
from re import Pattern
from socket import SocketKind, socket
from typing import Any, ClassVar

DEFAULT_TCP_LOGGING_PORT: int
DEFAULT_UDP_LOGGING_PORT: int
DEFAULT_HTTP_LOGGING_PORT: int
DEFAULT_SOAP_LOGGING_PORT: int
SYSLOG_UDP_PORT: int
SYSLOG_TCP_PORT: int

class WatchedFileHandler(FileHandler):
    dev: int  # undocumented
    ino: int  # undocumented
    if sys.version_info >= (3, 9):
        def __init__(
            self, filename: StrPath, mode: str = ..., encoding: str | None = ..., delay: bool = ..., errors: str | None = ...
        ) -> None: ...
    else:
        def __init__(self, filename: StrPath, mode: str = ..., encoding: str | None = ..., delay: bool = ...) -> None: ...

    def _statstream(self) -> None: ...  # undocumented
    def reopenIfNeeded(self) -> None: ...

class BaseRotatingHandler(FileHandler):
    namer: Callable[[str], str] | None
    rotator: Callable[[str, str], None] | None
    if sys.version_info >= (3, 9):
        def __init__(
            self, filename: StrPath, mode: str, encoding: str | None = ..., delay: bool = ..., errors: str | None = ...
        ) -> None: ...
    else:
        def __init__(self, filename: StrPath, mode: str, encoding: str | None = ..., delay: bool = ...) -> None: ...

    def rotation_filename(self, default_name: str) -> str: ...
    def rotate(self, source: str, dest: str) -> None: ...

class RotatingFileHandler(BaseRotatingHandler):
    maxBytes: str  # undocumented
    backupCount: int  # undocumented
    if sys.version_info >= (3, 9):
        def __init__(
            self,
            filename: StrPath,
            mode: str = ...,
            maxBytes: int = ...,
            backupCount: int = ...,
            encoding: str | None = ...,
            delay: bool = ...,
            errors: str | None = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            filename: StrPath,
            mode: str = ...,
            maxBytes: int = ...,
            backupCount: int = ...,
            encoding: str | None = ...,
            delay: bool = ...,
        ) -> None: ...

    def doRollover(self) -> None: ...
    def shouldRollover(self, record: LogRecord) -> int: ...  # undocumented

class TimedRotatingFileHandler(BaseRotatingHandler):
    when: str  # undocumented
    backupCount: int  # undocumented
    utc: bool  # undocumented
    atTime: datetime.time | None  # undocumented
    interval: int  # undocumented
    suffix: str  # undocumented
    dayOfWeek: int  # undocumented
    rolloverAt: int  # undocumented
    extMatch: Pattern[str]  # undocumented
    if sys.version_info >= (3, 9):
        def __init__(
            self,
            filename: StrPath,
            when: str = ...,
            interval: int = ...,
            backupCount: int = ...,
            encoding: str | None = ...,
            delay: bool = ...,
            utc: bool = ...,
            atTime: datetime.time | None = ...,
            errors: str | None = ...,
        ) -> None: ...
    else:
        def __init__(
            self,
            filename: StrPath,
            when: str = ...,
            interval: int = ...,
            backupCount: int = ...,
            encoding: str | None = ...,
            delay: bool = ...,
            utc: bool = ...,
            atTime: datetime.time | None = ...,
        ) -> None: ...

    def doRollover(self) -> None: ...
    def shouldRollover(self, record: LogRecord) -> int: ...  # undocumented
    def computeRollover(self, currentTime: int) -> int: ...  # undocumented
    def getFilesToDelete(self) -> list[str]: ...  # undocumented

class SocketHandler(Handler):
    host: str  # undocumented
    port: int | None  # undocumented
    address: tuple[str, int] | str  # undocumented
    sock: socket | None  # undocumented
    closeOnError: bool  # undocumented
    retryTime: float | None  # undocumented
    retryStart: float  # undocumented
    retryFactor: float  # undocumented
    retryMax: float  # undocumented
    def __init__(self, host: str, port: int | None) -> None: ...
    def makeSocket(self, timeout: float = ...) -> socket: ...  # timeout is undocumented
    def makePickle(self, record: LogRecord) -> bytes: ...
    def send(self, s: ReadableBuffer) -> None: ...
    def createSocket(self) -> None: ...

class DatagramHandler(SocketHandler):
    def makeSocket(self) -> socket: ...  # type: ignore[override]

class SysLogHandler(Handler):
    LOG_EMERG: int
    LOG_ALERT: int
    LOG_CRIT: int
    LOG_ERR: int
    LOG_WARNING: int
    LOG_NOTICE: int
    LOG_INFO: int
    LOG_DEBUG: int

    LOG_KERN: int
    LOG_USER: int
    LOG_MAIL: int
    LOG_DAEMON: int
    LOG_AUTH: int
    LOG_SYSLOG: int
    LOG_LPR: int
    LOG_NEWS: int
    LOG_UUCP: int
    LOG_CRON: int
    LOG_AUTHPRIV: int
    LOG_FTP: int

    if sys.version_info >= (3, 9):
        LOG_NTP: int
        LOG_SECURITY: int
        LOG_CONSOLE: int
        LOG_SOLCRON: int

    LOG_LOCAL0: int
    LOG_LOCAL1: int
    LOG_LOCAL2: int
    LOG_LOCAL3: int
    LOG_LOCAL4: int
    LOG_LOCAL5: int
    LOG_LOCAL6: int
    LOG_LOCAL7: int
    address: tuple[str, int] | str  # undocumented
    unixsocket: bool  # undocumented
    socktype: SocketKind  # undocumented
    ident: str  # undocumented
    append_nul: bool  # undocumented
    facility: int  # undocumented
    priority_names: ClassVar[dict[str, int]]  # undocumented
    facility_names: ClassVar[dict[str, int]]  # undocumented
    priority_map: ClassVar[dict[str, str]]  # undocumented
    def __init__(self, address: tuple[str, int] | str = ..., facility: int = ..., socktype: SocketKind | None = ...) -> None: ...
    if sys.version_info >= (3, 11):
        def createSocket(self) -> None: ...

    def encodePriority(self, facility: int | str, priority: int | str) -> int: ...
    def mapPriority(self, levelName: str) -> str: ...

class NTEventLogHandler(Handler):
    def __init__(self, appname: str, dllname: str | None = ..., logtype: str = ...) -> None: ...
    def getEventCategory(self, record: LogRecord) -> int: ...
    # TODO correct return value?
    def getEventType(self, record: LogRecord) -> int: ...
    def getMessageID(self, record: LogRecord) -> int: ...

class SMTPHandler(Handler):
    mailhost: str  # undocumented
    mailport: int | None  # undocumented
    username: str | None  # undocumented
    # password only exists as an attribute if passed credentials is a tuple or list
    password: str  # undocumented
    fromaddr: str  # undocumented
    toaddrs: list[str]  # undocumented
    subject: str  # undocumented
    secure: tuple[()] | tuple[str] | tuple[str, str] | None  # undocumented
    timeout: float  # undocumented
    def __init__(
        self,
        mailhost: str | tuple[str, int],
        fromaddr: str,
        toaddrs: str | list[str],
        subject: str,
        credentials: tuple[str, str] | None = ...,
        secure: tuple[()] | tuple[str] | tuple[str, str] | None = ...,
        timeout: float = ...,
    ) -> None: ...
    def getSubject(self, record: LogRecord) -> str: ...

class BufferingHandler(Handler):
    capacity: int  # undocumented
    buffer: list[LogRecord]  # undocumented
    def __init__(self, capacity: int) -> None: ...
    def shouldFlush(self, record: LogRecord) -> bool: ...

class MemoryHandler(BufferingHandler):
    flushLevel: int  # undocumented
    target: Handler | None  # undocumented
    flushOnClose: bool  # undocumented
    def __init__(self, capacity: int, flushLevel: int = ..., target: Handler | None = ..., flushOnClose: bool = ...) -> None: ...
    def setTarget(self, target: Handler | None) -> None: ...

class HTTPHandler(Handler):
    host: str  # undocumented
    url: str  # undocumented
    method: str  # undocumented
    secure: bool  # undocumented
    credentials: tuple[str, str] | None  # undocumented
    context: ssl.SSLContext | None  # undocumented
    def __init__(
        self,
        host: str,
        url: str,
        method: str = ...,
        secure: bool = ...,
        credentials: tuple[str, str] | None = ...,
        context: ssl.SSLContext | None = ...,
    ) -> None: ...
    def mapLogRecord(self, record: LogRecord) -> dict[str, Any]: ...
    if sys.version_info >= (3, 9):
        def getConnection(self, host: str, secure: bool) -> http.client.HTTPConnection: ...  # undocumented

class QueueHandler(Handler):
    queue: SimpleQueue[Any] | Queue[Any]  # undocumented
    def __init__(self, queue: SimpleQueue[Any] | Queue[Any]) -> None: ...
    def prepare(self, record: LogRecord) -> Any: ...
    def enqueue(self, record: LogRecord) -> None: ...

class QueueListener:
    handlers: tuple[Handler, ...]  # undocumented
    respect_handler_level: bool  # undocumented
    queue: SimpleQueue[Any] | Queue[Any]  # undocumented
    def __init__(self, queue: SimpleQueue[Any] | Queue[Any], *handlers: Handler, respect_handler_level: bool = ...) -> None: ...
    def dequeue(self, block: bool) -> LogRecord: ...
    def prepare(self, record: LogRecord) -> Any: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def enqueue_sentinel(self) -> None: ...
    def handle(self, record: LogRecord) -> None: ...



from typing import Any

from . import plugin_base

has_xdist: bool
py2k: Any

def pytest_addoption(parser) -> None: ...
def pytest_configure(config) -> None: ...

DUMP_PYANNOTATE: bool

def collect_types_fixture() -> None: ...
def pytest_sessionstart(session) -> None: ...
def pytest_sessionfinish(session) -> None: ...
def pytest_collection_finish(session): ...
def pytest_configure_node(node) -> None: ...
def pytest_testnodedown(node, error) -> None: ...
def pytest_collection_modifyitems(session, config, items): ...
def pytest_pycollect_makeitem(collector, name, obj): ...
def pytest_runtest_setup(item) -> None: ...
def pytest_runtest_teardown(item, nextitem) -> None: ...
def pytest_runtest_call(item) -> None: ...
def pytest_runtest_logreport(report) -> None: ...
def setup_class_methods(request) -> None: ...
def setup_test_methods(request) -> None: ...
def getargspec(fn): ...

class PytestFixtureFunctions(plugin_base.FixtureFunctions):
    def skip_test_exception(self, *arg, **kw): ...
    def mark_base_test_class(self): ...
    def combinations(self, *arg_sets, **kw): ...
    def param_ident(self, *parameters): ...
    def fixture(self, *arg, **kw): ...
    def get_current_test_name(self): ...
    def async_test(self, fn): ...



from typing import Any

from dateparser.parser import _parser

class CalendarBase:
    parser: Any
    source: Any
    def __init__(self, source) -> None: ...
    def get_date(self): ...

class non_gregorian_parser(_parser):
    calendar_converter: Any
    default_year: Any
    default_month: Any
    default_day: Any
    non_gregorian_date_cls: Any
    @classmethod
    def to_latin(cls, source): ...
    @classmethod
    def parse(cls, datestring, settings): ...



from ..console_menu import ExitItem as ExitItem, MenuItem as MenuItem
from .command_item import CommandItem as CommandItem
from .external_item import ExternalItem as ExternalItem
from .function_item import FunctionItem as FunctionItem
from .selection_item import SelectionItem as SelectionItem
from .submenu_item import SubmenuItem as SubmenuItem



#!/usr/bin/env python3

# For security (and simplicity) reasons, only a limited kind of files can be
# present in /stdlib and /stubs directories, see README for detail. Here we
# verify these constraints.
from __future__ import annotations

import os
import re
import sys
from pathlib import Path

import tomli
import yaml
from packaging.requirements import Requirement
from packaging.specifiers import SpecifierSet
from packaging.version import Version
from utils import VERSIONS_RE, get_all_testcase_directories, get_gitignore_spec, spec_matches_path, strip_comments

metadata_keys = {"version", "requires", "extra_description", "obsolete_since", "no_longer_updated", "tool"}
tool_keys = {"stubtest": {"skip", "apt_dependencies", "extras", "ignore_missing_stub"}}
extension_descriptions = {".pyi": "stub", ".py": ".py"}


def assert_consistent_filetypes(directory: Path, *, kind: str, allowed: set[str]) -> None:
    """Check that given directory contains only valid Python files of a certain kind."""
    allowed_paths = {Path(f) for f in allowed}
    contents = list(directory.iterdir())
    gitignore_spec = get_gitignore_spec()
    while contents:
        entry = contents.pop()
        if spec_matches_path(gitignore_spec, entry):
            continue
        if entry.relative_to(directory) in allowed_paths:
            # Note if a subdirectory is allowed, we will not check its contents
            continue
        if entry.is_file():
            assert entry.stem.isidentifier(), f'Files must be valid modules, got: "{entry}"'
            bad_filetype = f'Only {extension_descriptions[kind]!r} files allowed in the "{directory}" directory; got: {entry}'
            assert entry.suffix == kind, bad_filetype
        else:
            assert entry.name.isidentifier(), f"Directories must be valid packages, got: {entry}"
            contents.extend(entry.iterdir())


def check_stdlib() -> None:
    assert_consistent_filetypes(Path("stdlib"), kind=".pyi", allowed={"_typeshed/README.md", "VERSIONS"})


def check_stubs() -> None:
    gitignore_spec = get_gitignore_spec()
    for dist in Path("stubs").iterdir():
        if spec_matches_path(gitignore_spec, dist):
            continue
        assert dist.is_dir(), f"Only directories allowed in stubs, got {dist}"

        valid_dist_name = "^([A-Z0-9]|[A-Z0-9][A-Z0-9._-]*[A-Z0-9])$"  # courtesy of PEP 426
        assert re.fullmatch(
            valid_dist_name, dist.name, re.IGNORECASE
        ), f"Directory name must be a valid distribution name: {dist}"
        assert not dist.name.startswith("types-"), f"Directory name not allowed to start with 'types-': {dist}"

        allowed = {"METADATA.toml", "README", "README.md", "README.rst", "@tests"}
        assert_consistent_filetypes(dist, kind=".pyi", allowed=allowed)


def check_test_cases() -> None:
    for package_name, testcase_dir in get_all_testcase_directories():
        assert_consistent_filetypes(testcase_dir, kind=".py", allowed={"README.md"})
        bad_test_case_filename = 'Files in a `test_cases` directory must have names starting with "check_"; got "{}"'
        for file in testcase_dir.rglob("*.py"):
            assert file.stem.startswith("check_"), bad_test_case_filename.format(file)
            if package_name != "stdlib":
                with open(file, encoding="UTF-8") as f:
                    lines = {line.strip() for line in f}
                pyright_setting_not_enabled_msg = (
                    f'Third-party test-case file "{file}" must have '
                    f'"# pyright: reportUnnecessaryTypeIgnoreComment=true" '
                    f"at the top of the file"
                )
                has_pyright_setting_enabled = "# pyright: reportUnnecessaryTypeIgnoreComment=true" in lines
                assert has_pyright_setting_enabled, pyright_setting_not_enabled_msg


def check_no_symlinks() -> None:
    files = [os.path.join(root, file) for root, _, files in os.walk(".") for file in files]
    no_symlink = "You cannot use symlinks in typeshed, please copy {} to its link."
    for file in files:
        _, ext = os.path.splitext(file)
        if ext == ".pyi" and os.path.islink(file):
            raise ValueError(no_symlink.format(file))


def check_versions() -> None:
    versions = set()
    with open("stdlib/VERSIONS", encoding="UTF-8") as f:
        data = f.read().splitlines()
    for line in data:
        line = strip_comments(line)
        if line == "":
            continue
        m = VERSIONS_RE.match(line)
        if not m:
            raise AssertionError(f"Bad line in VERSIONS: {line}")
        module = m.group(1)
        assert module not in versions, f"Duplicate module {module} in VERSIONS"
        versions.add(module)
    modules = _find_stdlib_modules()
    # Sub-modules don't need to be listed in VERSIONS.
    extra = {m.split(".")[0] for m in modules} - versions
    assert not extra, f"Modules not in versions: {extra}"
    extra = versions - modules
    assert not extra, f"Versions not in modules: {extra}"


def _find_stdlib_modules() -> set[str]:
    modules = set()
    for path, _, files in os.walk("stdlib"):
        for filename in files:
            base_module = ".".join(os.path.normpath(path).split(os.sep)[1:])
            if filename == "__init__.pyi":
                modules.add(base_module)
            elif filename.endswith(".pyi"):
                mod, _ = os.path.splitext(filename)
                modules.add(f"{base_module}.{mod}" if base_module else mod)
    return modules


def check_metadata() -> None:
    for distribution in os.listdir("stubs"):
        with open(os.path.join("stubs", distribution, "METADATA.toml"), encoding="UTF-8") as f:
            data = tomli.loads(f.read())
        assert "version" in data, f"Missing version for {distribution}"
        version = data["version"]
        msg = f"Unsupported version {repr(version)}"
        assert isinstance(version, str), msg
        # Check that the version parses
        Version(version.removesuffix(".*"))
        for key in data:
            assert key in metadata_keys, f"Unexpected key {key} for {distribution}"
        assert isinstance(data.get("requires", []), list), f"Invalid requires value for {distribution}"
        for dep in data.get("requires", []):
            assert isinstance(dep, str), f"Invalid requirement {repr(dep)} for {distribution}"
            for space in " \t\n":
                assert space not in dep, f"For consistency, requirement should not have whitespace: {dep}"
            # Check that the requirement parses
            Requirement(dep)

        assert set(data.get("tool", [])).issubset(tool_keys.keys()), f"Unrecognised tool for {distribution}"
        for tool, tk in tool_keys.items():
            for key in data.get("tool", {}).get(tool, {}):
                assert key in tk, f"Unrecognised {tool} key {key} for {distribution}"


def get_txt_requirements() -> dict[str, SpecifierSet]:
    with open("requirements-tests.txt", encoding="UTF-8") as requirements_file:
        stripped_lines = map(strip_comments, requirements_file)
        requirements = map(Requirement, filter(None, stripped_lines))
        return {requirement.name: requirement.specifier for requirement in requirements}


def get_precommit_requirements() -> dict[str, SpecifierSet]:
    with open(".pre-commit-config.yaml", encoding="UTF-8") as precommit_file:
        precommit = precommit_file.read()
    yam = yaml.load(precommit, Loader=yaml.Loader)
    precommit_requirements = {}
    for repo in yam["repos"]:
        hook = repo["hooks"][0]
        package_name, package_rev = hook["id"], repo["rev"]
        package_specifier = SpecifierSet(f"=={package_rev.removeprefix('v')}")
        precommit_requirements[package_name] = package_specifier
        for additional_req in hook.get("additional_dependencies", []):
            req = Requirement(additional_req)
            precommit_requirements[req.name] = req.specifier
    return precommit_requirements


def check_requirements() -> None:
    requirements_txt_requirements = get_txt_requirements()
    precommit_requirements = get_precommit_requirements()
    no_txt_entry_msg = "All pre-commit requirements must also be listed in `requirements-tests.txt` (missing {requirement!r})"
    for requirement, specifier in precommit_requirements.items():
        assert requirement in requirements_txt_requirements, no_txt_entry_msg.format(requirement)
        specifier_mismatch = (
            f'Specifier "{specifier}" for {requirement!r} in `.pre-commit-config.yaml` '
            f'does not match specifier "{requirements_txt_requirements[requirement]}" in `requirements-tests.txt`'
        )
        assert specifier == requirements_txt_requirements[requirement], specifier_mismatch


if __name__ == "__main__":
    assert sys.version_info >= (3, 9), "Python 3.9+ is required to run this test"
    check_stdlib()
    check_versions()
    check_stubs()
    check_metadata()
    check_no_symlinks()
    check_test_cases()
    check_requirements()



from typing import Any
from typing_extensions import TypeAlias

# Enable when pyasn1 gets stubs:
# from pyasn1.type.univ import Sequence
_Sequence: TypeAlias = Any

class SicilyBindResponse(_Sequence):
    tagSet: Any
    componentType: Any

class DirSyncControlRequestValue(_Sequence):
    componentType: Any

class DirSyncControlResponseValue(_Sequence):
    componentType: Any

class SdFlags(_Sequence):
    componentType: Any

class ExtendedDN(_Sequence):
    componentType: Any

def dir_sync_control(criticality, object_security, ancestors_first, public_data_only, incremental_values, max_length, cookie): ...
def extended_dn_control(criticality: bool = ..., hex_format: bool = ...): ...
def show_deleted_control(criticality: bool = ...): ...
def security_descriptor_control(criticality: bool = ..., sdflags: int = ...): ...
def persistent_search_control(criticality: bool = ...): ...
